/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Access - OrgUnitArchiveService.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.dlcm.service;

import java.util.List;

import org.springframework.boot.autoconfigure.condition.ConditionalOnBean;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;

import ch.unige.solidify.rest.RestCollectionPage;
import ch.unige.solidify.service.CompositeNoSqlResourceService;

import ch.dlcm.DLCMConstants;
import ch.dlcm.controller.AccessController;
import ch.dlcm.model.MetadataVisibility;
import ch.dlcm.model.access.OrgUnit;
import ch.dlcm.model.index.ArchiveMetadata;
import ch.dlcm.service.rest.fallback.FallbackOrganizationalUnitRemoteResourceService;

@Service("OrgUnitArchiveService")
@ConditionalOnBean(AccessController.class)
public class OrgUnitArchiveService extends CompositeNoSqlResourceService<OrgUnit> {

  private final FallbackOrganizationalUnitRemoteResourceService organizationalUnitRemoteResourceService;

  private final SearchService searchService;

  public OrgUnitArchiveService(FallbackOrganizationalUnitRemoteResourceService organizationalUnitRemoteResourceService,
          SearchService searchService) {
    this.organizationalUnitRemoteResourceService = organizationalUnitRemoteResourceService;
    this.searchService = searchService;
  }

  @Override
  public boolean delete(OrgUnit orgUnit) {
    throw new UnsupportedOperationException();
  }

  @Override
  public List<OrgUnit> findAll(OrgUnit search) {
    return null;
  }

  @Override
  @SuppressWarnings({ "squid:S3776", "unchecked" })
  public <V> Page<V> findByParentId(String parentId, Class<?> parentClass, Class<V> childClass, Pageable pageable) {
    final OrgUnit orgUnit = this.findOne(parentId);
    if (childClass.equals(ArchiveMetadata.class)) {
      final Page<ArchiveMetadata> archives = this.searchService.search(
              MetadataVisibility.PUBLIC,
              DLCMConstants.AIP_ORGA_UNIT + ":" + orgUnit.getResId(),
              pageable);
      return (Page<V>) archives;

    }
    throw new UnsupportedOperationException(
            "Missing implementation 'findByParentId' method (" + parentClass.getSimpleName() + "->" + childClass.getSimpleName() + ")");
  }

  @Override
  @SuppressWarnings({ "squid:S3776", "unchecked" })
  public <V> V findByParentIdAndChildId(String parentId, Class<?> parentClass, Class<V> childClass, String id) {
    return (V) this.searchService.findOne(MetadataVisibility.PUBLIC, id);
  }

  @Override
  public OrgUnit findOne(String id) {
    final Page<ArchiveMetadata> archives = this.searchService.search(
            MetadataVisibility.PUBLIC,
            DLCMConstants.AIP_ORGA_UNIT + ":" + id,
            PageRequest.of(0, RestCollectionPage.MAX_SIZE_PAGE));
    return new OrgUnit(this.organizationalUnitRemoteResourceService.getOrgUnit(id), archives.getContent());
  }

  @Override
  public OrgUnit save(OrgUnit orgUnit) {
    throw new UnsupportedOperationException();
  }

  @Override
  public OrgUnit update(OrgUnit orgUnit) {
    throw new UnsupportedOperationException();
  }

}
