/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Admin - ResearchDomainService.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.dlcm.business;

import java.util.List;
import java.util.Optional;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.boot.autoconfigure.condition.ConditionalOnBean;
import org.springframework.stereotype.Service;

import ch.unige.solidify.service.ResourceService;

import ch.dlcm.controller.AdminController;
import ch.dlcm.model.settings.ResearchDomain;
import ch.dlcm.repository.ResearchDomainRepository;
import ch.dlcm.specification.ResearchDomainSpecification;

@Service
@ConditionalOnBean(AdminController.class)
public class ResearchDomainService extends ResourceService<ResearchDomain> {
  Logger log = LoggerFactory.getLogger(ResearchDomainService.class);

  private final ResearchDomainRepository researchDomainRepository;

  public ResearchDomainService(ResearchDomainRepository researchDomainRepository) {
    this.researchDomainRepository = researchDomainRepository;
  }

  public List<String> listSource() {
    return this.researchDomainRepository.listSource();
  }

  public Optional<ResearchDomain> findBySourceAndCode(String source, String code) {
    return this.researchDomainRepository.findBySourceAndCode(source, code);
  }

  @Override
  public ResearchDomainSpecification getSpecification(ResearchDomain resource) {
    return new ResearchDomainSpecification(resource);
  }

  public void initOptionalData() {
    // 1st level
    this.createResearchDomain("SNF", "00000", "Undefined domain");
    this.createResearchDomain("SNF", "10000", "Human and Social Sciences");
    this.createResearchDomain("SNF", "20000", "Mathematics, Natural- and Engineering Sciences");
    this.createResearchDomain("SNF", "30000", "Biology and Medicine");
  }

  private void createResearchDomain(String source, String code, String name) {
    if (!this.findBySourceAndCode(source, code).isPresent()) {
      ResearchDomain rd = new ResearchDomain();
      rd.setSource(source);
      rd.setCode(code);
      rd.setName(name);
      this.save(rd);
      this.log.info("Research Domain '{}-{}' created", source, code);
    }
  }
}
