/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Admin - SystemPropertyController.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.dlcm.controller.admin;

import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.linkTo;

import java.util.List;

import org.springframework.boot.autoconfigure.condition.ConditionalOnBean;
import org.springframework.hateoas.RepresentationModel;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import ch.unige.solidify.config.SolidifyProperties;
import ch.unige.solidify.controller.ControllerWithHateoasHome;
import ch.unige.solidify.model.index.FacetProperties;

import ch.dlcm.config.DLCMProperties;
import ch.dlcm.controller.AdminController;
import ch.dlcm.rest.UrlPath;
import ch.dlcm.service.rest.trusted.TrustedIndexFieldAliasRemoteResourceService;

@RestController
@ConditionalOnBean(AdminController.class)
@RequestMapping(UrlPath.ADMIN_SYSTEM_PROPERTY)
public class SystemPropertyController implements ControllerWithHateoasHome {

  private final DLCMProperties config;

  private final SystemProperties systemProperties;

  private TrustedIndexFieldAliasRemoteResourceService indexFieldAliasRemoteService;

  public SystemPropertyController(DLCMProperties dlcmProperties, SolidifyProperties solidifyProperties,
          TrustedIndexFieldAliasRemoteResourceService indexFieldAliasRemoteService) {
    this.config = dlcmProperties;
    this.systemProperties = new SystemProperties(dlcmProperties, solidifyProperties);
    this.indexFieldAliasRemoteService = indexFieldAliasRemoteService;
  }

  private static class SystemProperties extends RepresentationModel<SystemProperties> {
    private String defaultChecksum;
    private String defaultLicense;
    private String fileSizeLimit;
    private String orcidClientId;
    private String[] goldenFormats;
    private String[] forbiddenCharacters;
    private String[] checksums;
    private List<FacetProperties> searchFacets;

    public SystemProperties(DLCMProperties dlcmProperties, SolidifyProperties solidifyProperties) {
      this.defaultChecksum = dlcmProperties.getParameters().getDefaultChecksum();
      this.defaultLicense = dlcmProperties.getParameters().getDefaultLicense();
      this.fileSizeLimit = String.valueOf(dlcmProperties.getParameters().getFileSizeLimit().toBytes());
      this.orcidClientId = solidifyProperties.getOrcid().getClientId();
      this.goldenFormats = dlcmProperties.getParameters().getGoldenFormats();
      this.forbiddenCharacters = dlcmProperties.getParameters().getForbiddenCharacters();
      this.checksums = dlcmProperties.getParameters().getChecksumList();
    }

    public String getDefaultChecksum() {
      return this.defaultChecksum;
    }

    public String getDefaultLicense() {
      return this.defaultLicense;
    }

    public String getFileSizeLimit() {
      return this.fileSizeLimit;
    }

    public List<FacetProperties> getSearchFacets() {
      return this.searchFacets;
    }

    public String getOrcidClientId() {
      return this.orcidClientId;
    }

    public String[] getGoldenFormats() {
      return this.goldenFormats;
    }

    public String[] getForbiddenCharacters() {
      return this.forbiddenCharacters;
    }

    public String[] getChecksums() {
      return checksums;
    }

    public void setSearchFacets(List<FacetProperties> searchFacets) {
      this.searchFacets = searchFacets;
    }
  }

  @GetMapping
  public HttpEntity<SystemProperties> systemProperties() {
    List<FacetProperties> facetProperties = this.indexFieldAliasRemoteService.getFacetProperties(this.config.getIndexing().getIndexName());
    this.systemProperties.setSearchFacets(facetProperties);
    this.systemProperties.removeLinks();
    this.systemProperties.add(linkTo(this.getClass()).withSelfRel());
    return new ResponseEntity<>(this.systemProperties, HttpStatus.OK);
  }
}
