/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Admin - NotificationRepository.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.dlcm.repository;

import org.springframework.boot.autoconfigure.condition.ConditionalOnBean;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.repository.Modifying;
import org.springframework.data.jpa.repository.Query;
import org.springframework.stereotype.Repository;

import ch.unige.solidify.repository.SolidifyRepository;

import ch.dlcm.controller.AdminController;
import ch.dlcm.model.notification.Notification;
import ch.dlcm.model.notification.NotificationCategory;
import ch.dlcm.model.notification.NotificationStatus;
import ch.dlcm.model.notification.NotificationType;

@Repository
@ConditionalOnBean(AdminController.class)
public interface NotificationRepository extends SolidifyRepository<Notification> {

  @Query("SELECT DISTINCT n"
          + " FROM Notification n"
          + " JOIN n.notificationType nt"
          + " LEFT JOIN n.notifiedOrgUnit ou"
          + " LEFT JOIN ou.personRoles oupr"
          + " LEFT JOIN oupr.compositeKey.role r"
          + " LEFT JOIN nt.notifiedOrgUnitRole nour"
          + " LEFT JOIN nt.notifiedApplicationRole nar"
          + " WHERE ((nt.notifiedApplicationRole IS NOT NULL AND :userApplicationRoleLevel <= nar.level)"
          + "     OR (nt.notifiedOrgUnitRole IS NOT NULL AND r.level <= nour.level AND oupr.compositeKey.person.resId = :personId))"
          + " AND (n.notificationStatus = :notificationStatus OR :notificationStatus IS NULL)"
          + " AND (n.notificationType = :notificationType OR :notificationType IS NULL)"
          + " AND (nt.notificationCategory = :notificationCategory OR :notificationCategory IS NULL)"
          + " AND (n.notifiedOrgUnit.resId = :notifiedOrgUnitId OR :notifiedOrgUnitId IS NULL)")
  Page<Notification> findInboxNotificationByPersonAndStatus(String personId, NotificationStatus notificationStatus,
          NotificationType notificationType, NotificationCategory notificationCategory, String notifiedOrgUnitId, int userApplicationRoleLevel,
          Pageable pageable);

  @Query("SELECT DISTINCT n"
          + " FROM Notification n"
          + " JOIN n.notificationType nt"
          + " LEFT JOIN n.notifiedOrgUnit ou"
          + " LEFT JOIN ou.personRoles oupr"
          + " LEFT JOIN oupr.compositeKey.role r"
          + " LEFT JOIN nt.notifiedOrgUnitRole nour"
          + " LEFT JOIN nt.notifiedApplicationRole nar"
          + " WHERE ((nt.notifiedApplicationRole IS NOT NULL AND :userApplicationRoleLevel <= nar.level)"
          + "   OR (nt.notifiedOrgUnitRole IS NOT NULL AND r.level <= nour.level AND oupr.compositeKey.person.resId = :personId)"
          + "   OR (n.emitter.resId =:userId))"
          + " AND n.resId = :notificationId")
  Notification findInboxNotificationById(String personId, String userId, int userApplicationRoleLevel, String notificationId);

  Notification findByResId(String resID);

  @Modifying
  @Query("DELETE FROM Notification n WHERE n.notifiedOrgUnit.resId = :notifiedOrgUnitId")
  void deleteNotificationsByNotifiedOrgUnitId(String notifiedOrgUnitId);

  @Modifying
  @Query("DELETE FROM Notification n WHERE n.objectId = :objectId")
  void deleteNotificationsByObjectId(String objectId);
}
