/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Admin - PersonRepository.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.dlcm.repository;

import java.util.Optional;

import org.springframework.boot.autoconfigure.condition.ConditionalOnBean;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.repository.Query;
import org.springframework.stereotype.Repository;

import ch.unige.solidify.repository.SolidifyRepository;

import ch.dlcm.controller.AdminController;
import ch.dlcm.model.settings.OrganizationalUnit;
import ch.dlcm.model.settings.Person;

@Repository
@ConditionalOnBean(AdminController.class)
public interface PersonRepository extends SolidifyRepository<Person> {

  @Query("SELECT DISTINCT oupr.compositeKey.organizationalUnit"
          + " FROM Person p"
          + " JOIN p.organizationalUnitRoles oupr"
          + " WHERE oupr.compositeKey.person.resId = :personId")
  Page<OrganizationalUnit> findOrganizationalUnitsByPersonId(String personId, Pageable pageable);

  @Query("SELECT DISTINCT oupr.compositeKey.organizationalUnit"
          + " FROM Person p"
          + " JOIN p.organizationalUnitRoles oupr"
          + " WHERE oupr.compositeKey.person.resId = :personId"
          + " AND oupr.compositeKey.organizationalUnit.resId=:orgUnitId")
  OrganizationalUnit findOrganizationalUnitsByPersonId(String personId, String orgUnitId);

  @Query("SELECT DISTINCT oupr.compositeKey.organizationalUnit"
          + " FROM Person p"
          + " JOIN p.organizationalUnitRoles oupr"
          + " WHERE oupr.compositeKey.person.resId = :personId"
          + " AND (:orgUnitName IS NULL OR oupr.compositeKey.organizationalUnit.name LIKE %:orgUnitName%)"
          + " ORDER BY oupr.compositeKey.organizationalUnit.name")
  Page<OrganizationalUnit> findOrganizationalUnitsByPersonIdAndOrgUnitName(String personId, String orgUnitName, Pageable pageable);

  @Query("SELECT DISTINCT oupr.compositeKey.organizationalUnit"
          + " FROM Person p"
          + " JOIN p.organizationalUnitRoles oupr"
          + " WHERE oupr.compositeKey.person.resId = :personId"
          + " AND (:orgUnitName IS NULL OR oupr.compositeKey.organizationalUnit.name LIKE %:orgUnitName%)"
          + " AND oupr.compositeKey.organizationalUnit.openingDate <= CURRENT_DATE"
          + " AND (oupr.compositeKey.organizationalUnit.closingDate IS NULL OR oupr.compositeKey.organizationalUnit.closingDate > CURRENT_DATE)"
          + " ORDER BY oupr.compositeKey.organizationalUnit.name")
  Page<OrganizationalUnit> findOpenOrganizationalUnitsByPersonIdAndOrgUnitName(String personId, String orgUnitName, Pageable pageable);

  @Query("SELECT DISTINCT p FROM Person p"
          + " INNER JOIN User u ON u.person.resId = p.resId"
          + " WHERE p.firstName LIKE %:search% OR p.lastName LIKE %:search% OR p.orcid LIKE %:search%")
  Page<Person> searchAllWithUser(String search, Pageable pageable);

  Person findByFirstName(String firstName);

  Optional<Person> findByOrcid(String orcid);
}
