/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Admin - NotificationPermissionService.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.dlcm.service.security;

import java.security.Principal;

import org.springframework.boot.autoconfigure.condition.ConditionalOnBean;
import org.springframework.stereotype.Service;

import ch.unige.solidify.service.HttpRequestInfoProvider;

import ch.dlcm.controller.AdminController;
import ch.dlcm.model.notification.Notification;
import ch.dlcm.model.notification.NotificationType;
import ch.dlcm.model.security.User;
import ch.dlcm.repository.NotificationRepository;
import ch.dlcm.repository.UserRepository;

@Service
@ConditionalOnBean(AdminController.class)
public class NotificationPermissionService extends AbstractPermissionService {

  private final HttpRequestInfoProvider httpRequestInfoProvider;
  private final UserRepository userRepository;
  private final NotificationRepository notificationRepository;

  public NotificationPermissionService(UserRepository userRepository, NotificationRepository notificationRepository,
          HttpRequestInfoProvider httpRequestInfoProvider) {
    this.userRepository = userRepository;
    this.notificationRepository = notificationRepository;
    this.httpRequestInfoProvider = httpRequestInfoProvider;
  }

  public boolean isAllowedToCreate(Notification notification) {
    if (this.isRootOrAdminOrTrustedRole()) {
      return true;
    }
    NotificationType notificationType = notification.getNotificationType();
    if (notificationType.equals(NotificationType.ACCESS_DATASET_REQUEST) ||
            notificationType.equals(NotificationType.JOIN_ORGUNIT_REQUEST) ||
            notificationType.equals(NotificationType.CREATE_ORGUNIT_REQUEST) ||
            notificationType.equals(NotificationType.APPROVE_DISPOSAL_BY_ORGUNIT_REQUEST)) {
      return true;
    }
    return false;
  }

  public boolean isMyNotification(String notificationId) {
    final Principal principal = this.httpRequestInfoProvider.getPrincipal();
    final User currentUser = this.userRepository.findByExternalUid(principal.getName());
    String personId = currentUser.getPerson().getResId();
    int userApplicationRoleLevel = currentUser.getApplicationRole().getLevel();
    final Notification inboxNotification = this.notificationRepository.findInboxNotificationById(personId, currentUser.getResId(), userApplicationRoleLevel,
            notificationId);
    return inboxNotification != null;
  }

}
