/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Admin - OrganizationalUnitPermissionService.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.dlcm.service.security;

import java.util.Collection;

import org.springframework.boot.autoconfigure.condition.ConditionalOnBean;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Service;

import ch.unige.solidify.auth.model.AuthApplicationRole;

import ch.dlcm.business.OrganizationalUnitService;
import ch.dlcm.controller.AdminController;
import ch.dlcm.controller.DLCMControllerAction;
import ch.dlcm.model.OrganizationalUnitAwareResource;
import ch.dlcm.service.rest.trusted.TrustedPersonRemoteResourceService;

@Service
@ConditionalOnBean(AdminController.class)
public class OrganizationalUnitPermissionService extends AbstractPermissionWithOrgUnitService {

  private final OrganizationalUnitService organizationalUnitService;

  public OrganizationalUnitPermissionService(OrganizationalUnitService organizationalUnitService,
          TrustedPersonRemoteResourceService PersonRemoteResourceService) {
    super(PersonRemoteResourceService);
    this.organizationalUnitService = organizationalUnitService;
  }

  @Override
  protected boolean isManagerAllowed(OrganizationalUnitAwareResource existingResource, DLCMControllerAction action) {
    return true;
  }

  @Override
  protected boolean isStewardAllowed(OrganizationalUnitAwareResource existingResource, DLCMControllerAction action) {
    return this.isActionAllowedForEveryMember(action);
  }

  @Override
  protected boolean isApproverAllowed(OrganizationalUnitAwareResource existingResource, DLCMControllerAction action) {
    return this.isActionAllowedForEveryMember(action);
  }

  @Override
  protected boolean isCreatorAllowed(OrganizationalUnitAwareResource existingResource, DLCMControllerAction action) {
    return this.isActionAllowedForEveryMember(action);
  }

  @Override
  protected boolean isVisitorAllowed(OrganizationalUnitAwareResource existingResource, DLCMControllerAction action) {
    return this.isActionAllowedForEveryMember(action);
  }

  @Override
  boolean canBypassPermissionCheck() {
    /*
     * if user is ADMIN, bypass organizational unit's specific permissions
     */
    final Collection<? extends GrantedAuthority> authorities = SecurityContextHolder.getContext().getAuthentication().getAuthorities();
    for (final GrantedAuthority authority : authorities) {
      if (authority.toString().equals(AuthApplicationRole.ADMIN_ID)) {
        return true;
      }
    }

    return super.canBypassPermissionCheck();
  }

  private boolean isActionAllowedForEveryMember(DLCMControllerAction action) {
    return action == DLCMControllerAction.LIST_MEMBERS ||
            action == DLCMControllerAction.GET_MEMBER ||
            action == DLCMControllerAction.LIST_FORMS ||
            action == DLCMControllerAction.GET_FORM ||
            action == DLCMControllerAction.GET_CURRENT_FORM;
  }

  @Override
  OrganizationalUnitAwareResource getExistingResource(String resId) {
    return this.organizationalUnitService.findOne(resId);
  }
}
