/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Archival Storage - FixityMgmt.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.dlcm.service;

import java.util.NoSuchElementException;

import org.springframework.boot.autoconfigure.condition.ConditionalOnBean;
import org.springframework.jms.annotation.JmsListener;
import org.springframework.stereotype.Service;

import ch.dlcm.business.ArchivalInfoPackageService;
import ch.dlcm.controller.ArchivalStorageController;
import ch.dlcm.message.AipFixityMessage;

@Service
@ConditionalOnBean(ArchivalStorageController.class)
public class FixityMgmt {

  private final ArchivalInfoPackageService aipService;
  private final FixityService fixityService;

  public FixityMgmt(ArchivalInfoPackageService aipService, FixityService fixityService) {
    this.aipService = aipService;
    this.fixityService = fixityService;
  }

  @JmsListener(destination = "${dlcm.queue.aip-fixity}")
  public void processCheckFixity(AipFixityMessage aipFixityMessage) {
    final String aipId = aipFixityMessage.getResId();
    if(!this.aipService.existsById(aipId)) {
      throw new NoSuchElementException("No AIP with id " + aipId);
    }
    this.fixityService.checkFixity(aipFixityMessage.getResId(), aipFixityMessage.getFixityCheckLevel(), aipFixityMessage.getCreatedBy());
  }

}
