/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Client - CompositionClientService.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.dlcm.service;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.springframework.web.client.RestTemplate;

import ch.unige.solidify.SolidifyConstants;
import ch.unige.solidify.rest.Resource;
import ch.unige.solidify.rest.RestCollectionPage;
import ch.unige.solidify.util.CollectionTool;

import ch.dlcm.DLCMConstants;

public abstract class CompositionClientService<T extends Resource> extends ResourceBaseClientService<T> {
  protected static final String DLCM_ID_LABEL = DLCMConstants.DB_RES_ID;
  protected static final String DLCM_PARENT_ID_LABEL = SolidifyConstants.URL_PARENT_ID_FIELD;

  protected CompositionClientService(String resourcePath, Class<T> resourceClass, String module) {
    super(resourcePath, resourceClass, module);
  }

  public T findOne(String parentId, String resId) {
    final String url = this.getBaseUrl() + this.resourcePath;
    final Map<String, String> params = new HashMap<>();
    params.put(DLCM_PARENT_ID_LABEL, parentId);
    params.put(DLCM_ID_LABEL, resId);
    return this.getObjectFromUrl(url, params);
  }

  protected List<T> findAll(String parentId, Class<T> itemClass) {
    final String url = this.getBaseUrl() + this.resourcePath + "?size=" + RestCollectionPage.MAX_SIZE_PAGE;
    final Map<String, String> params = new HashMap<>();
    params.put(DLCM_PARENT_ID_LABEL, parentId);
    final RestTemplate restTemplate = this.restClientTool.getClient(this.getBaseUrl());
    final String jsonString = restTemplate.getForObject(url, String.class, params);
    return CollectionTool.getList(jsonString, itemClass);
  }

}
