/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Client - OrderClientService.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.dlcm.service.access;

import java.util.List;
import java.util.Map;

import org.springframework.http.HttpEntity;
import org.springframework.http.HttpMethod;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;

import ch.unige.solidify.SolidifyConstants;
import ch.unige.solidify.rest.ActionName;
import ch.unige.solidify.rest.Result;

import ch.dlcm.model.access.Order;
import ch.dlcm.rest.DLCMActionName;
import ch.dlcm.rest.ModuleName;
import ch.dlcm.rest.ResourceName;
import ch.dlcm.service.ResourceClientService;

@Service
public class OrderClientService extends ResourceClientService<Order> {

  private static final String ORDER_PATH = SolidifyConstants.URL_SEP + ResourceName.ORDER;

  public OrderClientService() {
    super(ORDER_PATH, Order.class, ModuleName.ACCESS);
  }

  @Override
  public List<Order> findAll() {
    return this.findAll(Order.class);
  }

  public Order save(String resId) {
    return this.doActionOnId(resId, ActionName.SAVE);
  }

  @Override
  public List<Order> searchByProperties(Map<String, String> properties) {
    return this.searchByProperties(properties, Order.class);
  }

  public void setSubmitStatus(String resId) {
    final String url = this.getBaseUrl() + this.resourcePath + SolidifyConstants.URL_SEP + "{resId}" + SolidifyConstants.URL_SEP
            + DLCMActionName.SUBMIT;
    final RestTemplate restTemplate = this.getRestClientTool().getClient(this.getBaseUrl());
    restTemplate.exchange(url, HttpMethod.POST, new HttpEntity<>(null), Result.class, resId);
  }
}
