/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Client - FundingAgencyClientService.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.dlcm.service.admin;

import java.nio.file.Path;
import java.util.List;
import java.util.Map;

import org.springframework.core.io.Resource;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;

import ch.unige.solidify.SolidifyConstants;

import ch.dlcm.model.IdentifierType;
import ch.dlcm.model.settings.FundingAgency;
import ch.dlcm.model.settings.OrganizationalUnit;
import ch.dlcm.rest.DLCMActionName;
import ch.dlcm.rest.ModuleName;
import ch.dlcm.rest.ResourceName;
import ch.dlcm.service.ResourceClientService;

@Service
public class FundingAgencyClientService extends ResourceClientService<FundingAgency> {

  private static final String FUNDING_AGENCY_PATH = SolidifyConstants.URL_SEP + ResourceName.FUNDING_AGENCY;

  /********************************/

  public FundingAgencyClientService() {
    super(FUNDING_AGENCY_PATH, FundingAgency.class, ModuleName.ADMIN);
  }

  public void addOrganizationalUnit(String fundingAgencyId, String unitId) {
    this.addItem(fundingAgencyId, ResourceName.ORG_UNIT, unitId, OrganizationalUnit.class);
  }

  /********************************/

  @Override
  public List<FundingAgency> findAll() {
    return this.findAll(FundingAgency.class);
  }

  public List<OrganizationalUnit> getOrganizationalUnits(String fundingAgencyId) {
    return this.findAllLinkedResources(fundingAgencyId, ResourceName.ORG_UNIT, OrganizationalUnit.class);
  }

  public void removeOrganizationalUnit(String fundingAgencyId, String unitId) {
    this.removeItem(fundingAgencyId, ResourceName.ORG_UNIT, unitId);
  }

  @Override
  public List<FundingAgency> searchByProperties(Map<String, String> properties) {
    return this.searchByProperties(properties, FundingAgency.class);
  }

  public FundingAgency uploadLogo(String resId, Resource file) {
    final String url = this.getBaseUrl() + this.resourcePath + SolidifyConstants.URL_SEP + resId + SolidifyConstants.URL_SEP
            + DLCMActionName.UPLOAD_LOGO;
    return this.uploadFileWithUrl(url, file, FundingAgency.class);
  }

  public void downloadLogo(String resId, Path path) {
    final String url = this.resourcePath + SolidifyConstants.URL_SEP + resId + SolidifyConstants.URL_SEP + DLCMActionName.DOWNLOAD_LOGO;
    this.downloadByUrl(url, path);
  }

  public void deleteLogo(String resId) {
    final String url = this.resourcePath + SolidifyConstants.URL_SEP + resId + SolidifyConstants.URL_SEP + DLCMActionName.DELETE_LOGO;
    final RestTemplate restTemplate = this.getRestClientTool().getClient(this.getBaseUrl());
    restTemplate.delete(url);
  }

  public FundingAgency findByRorId(String rorId) {
    return this.findByIdentifierType(IdentifierType.ROR_ID, rorId);
  }

  public FundingAgency findByAcronym(String acronym) {
    return this.findByIdentifierType(IdentifierType.ACRONYM, acronym);
  }
}
