/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Client - InstitutionClientService.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.dlcm.service.admin;

import java.nio.file.Path;
import java.util.List;
import java.util.Map;

import org.springframework.core.io.Resource;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;

import ch.unige.solidify.SolidifyConstants;

import ch.dlcm.model.IdentifierType;
import ch.dlcm.model.settings.Institution;
import ch.dlcm.model.settings.OrganizationalUnit;
import ch.dlcm.model.settings.Person;
import ch.dlcm.rest.DLCMActionName;
import ch.dlcm.rest.ModuleName;
import ch.dlcm.rest.ResourceName;
import ch.dlcm.service.ResourceClientService;

@Service
public class InstitutionClientService extends ResourceClientService<Institution> {

  private static final String INSTITUTION_PATH = SolidifyConstants.URL_SEP + ResourceName.INSTITUTION;

  public InstitutionClientService() {
    super(INSTITUTION_PATH, Institution.class, ModuleName.ADMIN);
  }

  public void addPerson(String institutionId, String personId) {
    this.addItem(institutionId, ResourceName.PERSON, personId, Person.class);
  }

  @Override
  public List<Institution> findAll() {
    return this.findAll(Institution.class);
  }

  public List<OrganizationalUnit> getOrganizationalUnits(String institutionId) {
    return this.findAllLinkedResources(institutionId, ResourceName.ORG_UNIT, OrganizationalUnit.class);
  }

  public List<Person> getPersons(String institutionId) {
    return this.findAllLinkedResources(institutionId, ResourceName.PERSON, Person.class);
  }

  public void removeOrganizationalUnit(String institutionId, String organizationalUnitId) {
    this.removeItem(institutionId, ResourceName.ORG_UNIT, organizationalUnitId);
  }

  public void removePerson(String institutionId, String personId) {
    this.removeItem(institutionId, ResourceName.PERSON, personId);
  }

  @Override
  public List<Institution> searchByProperties(Map<String, String> properties) {
    return this.searchByProperties(properties, Institution.class);
  }

  public Institution uploadLogo(String resId, Resource file) {
    final String url = this.getBaseUrl() + this.resourcePath + SolidifyConstants.URL_SEP + resId + SolidifyConstants.URL_SEP
            + DLCMActionName.UPLOAD_LOGO;
    return this.uploadFileWithUrl(url, file, Institution.class);
  }

  public void downloadLogo(String resId, Path path) {
    final String url = this.resourcePath + SolidifyConstants.URL_SEP + resId + SolidifyConstants.URL_SEP + DLCMActionName.DOWNLOAD_LOGO;
    this.downloadByUrl(url, path);
  }

  public void deleteLogo(String resId) {
    final String url = this.resourcePath + SolidifyConstants.URL_SEP + resId + SolidifyConstants.URL_SEP + DLCMActionName.DELETE_LOGO;
    final RestTemplate restTemplate = this.getRestClientTool().getClient(this.getBaseUrl());
    restTemplate.delete(url);
  }

  public Institution findByRorId(String rorId) {
    return this.findByIdentifierType(IdentifierType.ROR_ID, rorId);
  }

  public Institution findByName(String name) {
    return this.findByIdentifierType(IdentifierType.NAME, name);
  }
}
