/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Data Management - ArchiveThumbnailController.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.dlcm.controller.datamgmt;

import java.io.IOException;
import java.util.Map;

import org.springframework.boot.autoconfigure.condition.ConditionalOnBean;
import org.springframework.data.domain.Pageable;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.method.annotation.StreamingResponseBody;

import ch.unige.solidify.SolidifyConstants;
import ch.unige.solidify.rest.RestCollection;
import ch.unige.solidify.security.EveryonePermissions;
import ch.unige.solidify.security.TrustedUserPermissions;

import ch.dlcm.DLCMConstants;
import ch.dlcm.controller.DLCMResourceWithLogoController;
import ch.dlcm.controller.DataMgmtController;
import ch.dlcm.model.index.ArchiveThumbnail;
import ch.dlcm.rest.DLCMActionName;
import ch.dlcm.rest.UrlPath;

@TrustedUserPermissions
@RestController
@ConditionalOnBean(DataMgmtController.class)
@RequestMapping(UrlPath.DATA_MGMT_ARCHIVE_THUMBNAIL)
public class ArchiveThumbnailController extends DLCMResourceWithLogoController<ArchiveThumbnail> {

  @Override
  @SuppressWarnings("squid:S4684")
  public HttpEntity<ArchiveThumbnail> create(@RequestBody ArchiveThumbnail archiveThumbnail) {
    return super.create(archiveThumbnail);
  }

  @Override
  public HttpEntity<ArchiveThumbnail> get(@PathVariable String id) {
    return super.get(id);
  }

  @Override
  public HttpEntity<RestCollection<ArchiveThumbnail>> list(@ModelAttribute ArchiveThumbnail search, Pageable pageable) {
    return super.list(search, pageable);
  }

  @Override
  public HttpEntity<ArchiveThumbnail> update(@PathVariable String id, @RequestBody Map<String, Object> updateMap) {
    return super.update(id, updateMap);
  }

  @Override
  public HttpStatus delete(@PathVariable String id) {
    return super.delete(id);
  }

  @PostMapping(SolidifyConstants.URL_ID_PLUS_SEP + DLCMActionName.UPLOAD_THUMBNAIL)
  @Override
  public HttpEntity<ArchiveThumbnail> uploadLogo(@PathVariable String id, @RequestParam(DLCMConstants.FILE) MultipartFile file) throws IOException {
    return super.uploadLogo(id, file);
  }

  @EveryonePermissions
  @GetMapping(SolidifyConstants.URL_ID_PLUS_SEP + DLCMActionName.DOWNLOAD_THUMBNAIL)
  @ResponseBody
  @Override
  public HttpEntity<StreamingResponseBody> downloadLogo(@PathVariable String id) throws IOException {
    return super.downloadLogo(id);
  }

  @DeleteMapping(SolidifyConstants.URL_ID_PLUS_SEP + DLCMActionName.DELETE_THUMBNAIL)
  @ResponseBody
  @Override
  public HttpEntity<ArchiveThumbnail> deleteLogo(@PathVariable String id) {
    return super.deleteLogo(id);
  }

}
