/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Data Management - ItemController.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.dlcm.controller.datamgmt;

import org.springframework.boot.autoconfigure.condition.ConditionalOnBean;
import org.springframework.data.domain.Pageable;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import ch.unige.solidify.controller.IndexResourceController;
import ch.unige.solidify.index.indexing.IndexingService;
import ch.unige.solidify.rest.RestCollection;
import ch.unige.solidify.security.TrustedUserPermissions;

import ch.dlcm.config.DLCMProperties;
import ch.dlcm.controller.DataMgmtController;
import ch.dlcm.model.index.RegistryMetadata;
import ch.dlcm.rest.UrlPath;

@TrustedUserPermissions
@RestController
@ConditionalOnBean(DataMgmtController.class)
@RequestMapping(UrlPath.DATA_MGMT_ITEM)
public class ItemController extends IndexResourceController<String, RegistryMetadata> {

  private final DLCMProperties config;

  public ItemController(IndexingService<RegistryMetadata> noSqlResourceService, DLCMProperties config) {
    super(noSqlResourceService);
    this.config = config;
  }

  @Override
  protected String getIndex() {
    return this.config.getIndexing().getRegistryName();
  }

  // **********************************
  // Override methods to apply security
  // **********************************

  @Override
  public HttpEntity<RegistryMetadata> create(@RequestBody RegistryMetadata t) {
    return super.create(t);
  }

  @Override
  public HttpEntity<RegistryMetadata> get(@PathVariable String id) {
    return super.get(id);
  }

  @Override
  public HttpEntity<RestCollection<RegistryMetadata>> list(@ModelAttribute RegistryMetadata search, Pageable pageable) {
    return super.list(search, pageable);
  }

  @Override
  public HttpEntity<RegistryMetadata> update(@PathVariable String id, @RequestBody RegistryMetadata t2) {
    return super.update(id, t2);
  }

  @Override
  public HttpStatus delete(@PathVariable String id) {
    return super.delete(id);
  }

  @Override
  public HttpStatus deleteList(@RequestBody String[] ids) {
    return super.deleteList(ids);
  }

}
