/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Data Management - InMemoryArchiveMetadataService.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.dlcm.service.inmemory;

import org.springframework.boot.autoconfigure.condition.ConditionalOnBean;
import org.springframework.context.annotation.Profile;
import org.springframework.stereotype.Service;

import ch.unige.solidify.IndexProperties;
import ch.unige.solidify.index.indexing.inmemory.InMemoryService;
import ch.unige.solidify.index.indexing.inmemory.IndexData;
import ch.unige.solidify.index.settings.inmemory.InMemorySettingsService;

import ch.dlcm.config.DLCMProperties;
import ch.dlcm.controller.DataMgmtController;
import ch.dlcm.model.index.ArchiveMetadata;

@Profile("index-inmemory")
@Service
@ConditionalOnBean(DataMgmtController.class)
public class InMemoryArchiveMetadataService extends InMemoryService<ArchiveMetadata> {

  public InMemoryArchiveMetadataService(
          IndexProperties indexProperties,
          DLCMProperties dlcmProperties,
          InMemorySettingsService settingsService,
          IndexData indexData) {
    super(indexProperties, settingsService, dlcmProperties.getIndexing().getIndexDefinitionList(), indexData);
  }

  @Override
  protected ArchiveMetadata getMetadataObject() {
    return new ArchiveMetadata();
  }

  @Override
  public String getIndexName(String indexName) {
    return indexName;
  }

}
