/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Integration Tests - AbstractAdminIT.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.dlcm.test.admin;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;

import ch.unige.solidify.model.index.IndexFieldAlias;

import ch.dlcm.DLCMConstants;
import ch.dlcm.model.notification.Notification;
import ch.dlcm.model.policies.DisseminationPolicy;
import ch.dlcm.model.policies.PreservationPolicy;
import ch.dlcm.model.policies.SubmissionPolicy;
import ch.dlcm.model.security.User;
import ch.dlcm.model.settings.ArchiveType;
import ch.dlcm.model.settings.FundingAgency;
import ch.dlcm.model.settings.Institution;
import ch.dlcm.model.settings.License;
import ch.dlcm.model.settings.MetadataType;
import ch.dlcm.model.settings.OrganizationalUnit;
import ch.dlcm.model.settings.Person;
import ch.dlcm.model.settings.ResearchDomain;
import ch.dlcm.model.settings.SubmissionAgreement;
import ch.dlcm.service.admin.ArchiveACLClientService;
import ch.dlcm.service.admin.ArchiveUserRatingClientService;
import ch.dlcm.service.admin.FundingAgencyClientService;
import ch.dlcm.service.admin.GlobalBannerClientService;
import ch.dlcm.service.admin.InstitutionClientService;
import ch.dlcm.service.admin.NotificationClientService;
import ch.dlcm.service.admin.ResearchDomainClientService;
import ch.dlcm.service.admin.SubmissionAgreementClientService;
import ch.dlcm.test.AbstractIT;

public abstract class AbstractAdminIT extends AbstractIT {

  @Autowired
  protected FundingAgencyClientService fundingAgencyService;

  @Autowired
  protected InstitutionClientService institutionService;

  @Autowired
  protected ResearchDomainClientService researchDomainService;

  @Autowired
  protected SubmissionAgreementClientService submissionAgreementService;

  @Autowired
  protected GlobalBannerClientService globalBannerService;

  @Autowired
  NotificationClientService notificationService;

  @Autowired
  ArchiveACLClientService archiveACLService;

  @Autowired
  protected ArchiveUserRatingClientService archiveUserRatingService;

  /***********************************/

  protected Person getLinkedPersonToUser(User user) {
    return this.personService.findOne(user.getPerson().getResId());
  }

  protected void clearDisseminationPolicyFixtures() {
    final List<DisseminationPolicy> dpList = this.disseminationPolicyService.findAll();

    for (final DisseminationPolicy dp : dpList) {
      if (dp.getName().startsWith(DLCMConstants.TEMPORARY_TEST_DATA_LABEL)) {
        this.disseminationPolicyService.delete(dp.getResId());
      }
    }
  }

  protected void clearFundingAgencyFixtures() {
    final List<FundingAgency> agencies = this.fundingAgencyService.findAll();

    for (final FundingAgency a : agencies) {

      if (a.getName().startsWith(DLCMConstants.TEMPORARY_TEST_DATA_LABEL)) {

        /*
         * Remove eventual links with OrganizationalUnits first
         */
        final List<OrganizationalUnit> child_units = this.fundingAgencyService.getOrganizationalUnits(a.getResId());
        if (child_units != null) {
          for (final OrganizationalUnit u : child_units) {
            this.fundingAgencyService.removeOrganizationalUnit(a.getResId(), u.getResId());
          }
        }

        this.fundingAgencyService.delete(a.getResId());
      }
    }
  }

  protected void clearInstitutionsFixtures() {

    final List<Institution> institutions = this.institutionService.findAll();

    for (final Institution institution : institutions) {

      if (institution.getName().startsWith(DLCMConstants.TEMPORARY_TEST_DATA_LABEL)) {

        /*
         * Remove eventual links with people first
         */
        final List<Person> people = this.institutionService.getPersons(institution.getResId());
        if (people != null && !people.isEmpty()) {
          for (final Person person : people) {
            this.institutionService.removePerson(institution.getResId(), person.getResId());
          }
        }

        /*
         * Remove eventual links with organizational units first
         */
        final List<OrganizationalUnit> units = this.institutionService.getOrganizationalUnits(institution.getResId());
        if (units != null && !units.isEmpty()) {
          for (final OrganizationalUnit unit : units) {
            this.institutionService.removeOrganizationalUnit(institution.getResId(), unit.getResId());
          }
        }

        this.institutionService.delete(institution.getResId());
      }
    }
  }

  protected void clearLicenceFixtures() {
    final List<License> licenses = this.licenseService.findAll();

    for (final License license : licenses) {
      if (license.getTitle().startsWith(DLCMConstants.TEMPORARY_TEST_DATA_LABEL)) {
        this.licenseService.delete(license.getResId());
      }
    }
  }

  protected void clearMetadataTypeFixtures() {
    final List<MetadataType> mtList = this.metadataTypeService.findAll();

    for (final MetadataType mt : mtList) {
      if (mt.getName().startsWith(DLCMConstants.TEMPORARY_TEST_DATA_LABEL)) {
        this.metadataTypeService.delete(mt.getResId());
      }
    }
  }

  protected void clearNotificationFixtures() {
    final List<Notification> notificationList = this.notificationService.findAll();

    for (final Notification notification : notificationList) {
      if (notification.getMessage().startsWith(DLCMConstants.TEMPORARY_TEST_DATA_LABEL)) {
        this.notificationService.delete(notification.getResId());
      }
    }
  }

  protected void clearOrganizationalUnitFixtures() {
    this.organizationalUnitService.findAll().stream()
            .filter(orgUnit -> orgUnit.getName().startsWith(DLCMConstants.TEMPORARY_TEST_DATA_LABEL))
            .forEach(orgUnit -> this.organizationalUnitService.delete(orgUnit.getResId()));
  }

  protected void clearPreservationPolicyFixtures() {
    final List<PreservationPolicy> ppList = this.preservationPolicyService.findAll();

    for (final PreservationPolicy pp : ppList) {
      if (pp.getName().startsWith(DLCMConstants.TEMPORARY_TEST_DATA_LABEL)) {
        this.preservationPolicyService.delete(pp.getResId());
      }
    }
  }

  protected void clearResearchDomainFixtures() {
    final List<ResearchDomain> rdList = this.researchDomainService.findAll();

    for (final ResearchDomain rd : rdList) {
      if (rd.getName().startsWith(DLCMConstants.TEMPORARY_TEST_DATA_LABEL)) {
        this.researchDomainService.delete(rd.getResId());
      }
    }
  }

  protected void clearArchiveTypeFixtures() {
    final List<ArchiveType> rdList = this.archiveTypeService.findAll();

    for (final ArchiveType at : rdList) {
      if (at.getTypeName().startsWith(DLCMConstants.TEMPORARY_TEST_DATA_LABEL)) {
        this.archiveTypeService.delete(at.getResId());
      }
    }
  }

  protected void clearSubmissionAgreementFixtures() {
    final List<SubmissionAgreement> agreements = this.submissionAgreementService.findAll();

    for (final SubmissionAgreement sa : agreements) {
      if (sa.getTitle().startsWith(DLCMConstants.TEMPORARY_TEST_DATA_LABEL)) {
        this.submissionAgreementService.delete(sa.getResId());
      }
    }
  }

  protected void clearSubmissionPolicyFixtures() {
    final List<SubmissionPolicy> spList = this.submissionPolicyService.findAll();

    for (final SubmissionPolicy sp : spList) {
      if (sp.getName().startsWith(DLCMConstants.TEMPORARY_TEST_DATA_LABEL)) {
        this.submissionPolicyService.delete(sp.getResId());
      }
    }
  }

  protected void clearIndexFieldAliasFixtures() {
    final List<IndexFieldAlias> indexFieldAliases = this.indexFieldAliasService.findAll();

    for (final IndexFieldAlias indexFieldAlias : indexFieldAliases) {
      if (indexFieldAlias.getIndexName().equals(DLCMConstants.TEST_RES_ID)) {
        this.indexFieldAliasService.delete(indexFieldAlias.getResId());
      }
    }
  }

  /***********************************/

  protected void clearUserFixtures() {
    final List<User> users = this.userService.findAll();

    for (final User u : users) {

      if (u.getFirstName().startsWith(DLCMConstants.TEMPORARY_TEST_DATA_LABEL)) {

        this.userService.delete(u.getResId());
      }
    }
  }

  protected void clearGlobalBannerFixtures() {
    this.globalBannerService.findAll().stream()
            .filter(globalBanner -> globalBanner.getName().startsWith(DLCMConstants.TEMPORARY_TEST_DATA_LABEL))
            .forEach(globalBanner -> this.globalBannerService.delete(globalBanner.getResId()));
  }
}
