/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Integration Tests - SubmissionPolicyIT.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.dlcm.test.admin;

import static org.junit.jupiter.api.Assertions.assertEquals;

import org.junit.jupiter.api.Test;

import ch.dlcm.model.policies.SubmissionPolicy;
import ch.dlcm.test.DLCMTestConstants;

class SubmissionPolicyIT extends AbstractAdminIT {

  @Override
  protected void setUser() {
    this.restClientTool.sudoAdmin();
  }

  @Test
  void creationTest() {
    // Create a policy
    final SubmissionPolicy policy1 = new SubmissionPolicy();
    policy1.setName(DLCMTestConstants.getRandomNameWithTemporaryLabel("submission policy "));
    policy1.setSubmissionApproval(true);
    final SubmissionPolicy policy2 = this.submissionPolicyService.create(policy1);

    // Test the creation
    assertEquals(policy1.getName(), policy2.getName());
    assertEquals(policy1.getSubmissionApproval(), policy2.getSubmissionApproval());
  }

  @Test
  void updateTest() {
    // Create a policy
    SubmissionPolicy policy1 = new SubmissionPolicy();
    policy1.setName(DLCMTestConstants.getRandomNameWithTemporaryLabel("A"));
    policy1.setSubmissionApproval(true);
    policy1 = this.submissionPolicyService.create(policy1);

    // Update the institution
    policy1.setName(DLCMTestConstants.getRandomNameWithTemporaryLabel("B"));
    policy1.setSubmissionApproval(false);
    this.submissionPolicyService.update(policy1.getResId(), policy1);
    final SubmissionPolicy policy2 = this.submissionPolicyService.findOne(policy1.getResId());

    // Test the update
    assertEquals(policy1.getName(), policy2.getName());
    assertEquals(policy1.getSubmissionApproval(), policy2.getSubmissionApproval());
    this.assertEqualsWithoutNanoSeconds(policy1.getCreationTime(), policy2.getCreationTime());
  }

  @Override
  protected void deleteFixtures() {
    this.clearOrganizationalUnitFixtures();
    this.clearSubmissionPolicyFixtures();
  }
}
