/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Integration Tests - CollectionsDemoIT.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.dlcm.test.demo;

import static org.junit.jupiter.api.Assertions.assertDoesNotThrow;
import static org.junit.jupiter.api.Assertions.assertNotNull;

import java.util.Map;

import org.junit.jupiter.api.Disabled;
import org.junit.jupiter.api.Order;
import org.junit.jupiter.api.Test;

import ch.dlcm.DLCMMetadataVersion;
import ch.dlcm.model.policies.PreservationPolicy;
import ch.dlcm.model.policies.SubmissionPolicy;
import ch.dlcm.model.settings.OrganizationalUnit;

@Order(5)
@Disabled(DLCMDemoConstants.DEMO_MESSAGE)
class CollectionsOfCollectionsDemoIT extends AbstractPreingestDemoIT {

  @Test
  void createBodmerCollectionsOfCollectionsForDemo() {
    final OrganizationalUnit bodmerOrgUnit = this.getOrgUnit(DLCMDemoConstants.BODMER_ORG_UNIT);
    final SubmissionPolicy submissionPolicy = this.submissionPolicyService
            .searchByProperties(Map.of("name", DLCMDemoConstants.DEMO_SUBMISSION_POLICY)).get(0);
    final PreservationPolicy preservationPolicy = this.preservationPolicyService
            .searchByProperties(Map.of("name", DLCMDemoConstants.DEMO_PRESERVATION_POLICY)).get(0);

    assertNotNull(bodmerOrgUnit, "Cannot find orgUnit: " + DLCMDemoConstants.BODMER_ORG_UNIT);
    assertNotNull(submissionPolicy, "Cannot find submission policy: " + DLCMDemoConstants.DEMO_SUBMISSION_POLICY);
    assertNotNull(preservationPolicy, "Cannot find preservation policy: " + DLCMDemoConstants.DEMO_PRESERVATION_POLICY);
    for (String depositName : this.getDepositList()) {
      final String collectionName = depositName + DLCMDemoConstants.COLLECTION + " of " + DLCMDemoConstants.COLLECTION;
      for (final DLCMMetadataVersion version : DLCMMetadataVersion.values()) {
        assertDoesNotThrow(() -> {
          this.createCollection(bodmerOrgUnit, version, collectionName, submissionPolicy, preservationPolicy, depositName, false, true);
        });
      }
    }
  }

  @Test
  void createBodmerHydridCollectionsForDemo() {
    final OrganizationalUnit bodmerOrgUnit = this.getOrgUnit(DLCMDemoConstants.BODMER_ORG_UNIT);
    final SubmissionPolicy submissionPolicy = this.submissionPolicyService
            .searchByProperties(Map.of("name", DLCMDemoConstants.DEMO_SUBMISSION_POLICY)).get(0);
    final PreservationPolicy preservationPolicy = this.preservationPolicyService
            .searchByProperties(Map.of("name", DLCMDemoConstants.DEMO_PRESERVATION_POLICY)).get(0);

    assertNotNull(bodmerOrgUnit, "Cannot find orgUnit: " + DLCMDemoConstants.BODMER_ORG_UNIT);
    assertNotNull(submissionPolicy, "Cannot find submission policy: " + DLCMDemoConstants.DEMO_SUBMISSION_POLICY);
    assertNotNull(preservationPolicy, "Cannot find preservation policy: " + DLCMDemoConstants.DEMO_PRESERVATION_POLICY);
    for (String depositName : this.getDepositList()) {
      final String collectionName = depositName + " Hybrid " + DLCMDemoConstants.COLLECTION;
      for (final DLCMMetadataVersion version : DLCMMetadataVersion.values()) {
        assertDoesNotThrow(() -> {
          createCollection(bodmerOrgUnit, version, collectionName, submissionPolicy, preservationPolicy, depositName, true, true);
        });
      }
    }
  }

}
