/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Integration Tests - DepositsDemoIT.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.dlcm.test.demo;

import static org.junit.jupiter.api.Assertions.assertDoesNotThrow;
import static org.junit.jupiter.api.Assertions.assertNotNull;
import static org.junit.jupiter.api.Assertions.assertTrue;

import java.io.IOException;
import java.util.Map;

import org.junit.jupiter.api.Disabled;
import org.junit.jupiter.api.Order;
import org.junit.jupiter.api.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.core.io.Resource;

import ch.dlcm.DLCMMetadataVersion;
import ch.dlcm.model.policies.PreservationPolicy;
import ch.dlcm.model.policies.SubmissionPolicy;
import ch.dlcm.model.preingest.Deposit;
import ch.dlcm.model.settings.OrganizationalUnit;

@Order(3)
@Disabled(DLCMDemoConstants.DEMO_MESSAGE)
class DepositsDemoIT extends AbstractPreingestDemoIT {

  private static final Logger log = LoggerFactory.getLogger(DepositsDemoIT.class);

  // @Test
  void createBodmerDeposits() {
    final OrganizationalUnit bodmerOrgUnit = this.getOrgUnit(DLCMDemoConstants.BODMER_ORG_UNIT);
    assertNotNull(bodmerOrgUnit, "Cannot find orgUnit: " + DLCMDemoConstants.BODMER_ORG_UNIT);
    for (final DLCMMetadataVersion version : DLCMMetadataVersion.values()) {
      assertDoesNotThrow(() -> {
        createDeposit(bodmerOrgUnit, version, null, null, BODMER_DEPOSIT_FOLDER, DLCMDemoConstants.ZIP_FILES);
      });
    }
  }

  @Test
  void createBodmerDepositsForDemo() {
    final OrganizationalUnit bodmerOrgUnit = this.getOrgUnit(DLCMDemoConstants.BODMER_ORG_UNIT);
    final SubmissionPolicy submissionPolicy = this.submissionPolicyService
            .searchByProperties(Map.of("name", DLCMDemoConstants.DEMO_SUBMISSION_POLICY)).get(0);
    final PreservationPolicy preservationPolicy = this.preservationPolicyService
            .searchByProperties(Map.of("name", DLCMDemoConstants.DEMO_PRESERVATION_POLICY)).get(0);

    assertNotNull(bodmerOrgUnit, "Cannot find orgUnit: " + DLCMDemoConstants.BODMER_ORG_UNIT);
    assertNotNull(submissionPolicy, "Cannot find submission policy: " + DLCMDemoConstants.DEMO_SUBMISSION_POLICY);
    assertNotNull(preservationPolicy, "Cannot find preservation policy: " + DLCMDemoConstants.DEMO_PRESERVATION_POLICY);
    for (final DLCMMetadataVersion version : DLCMMetadataVersion.values()) {
      assertDoesNotThrow(() -> {
        createDeposit(bodmerOrgUnit, version, submissionPolicy, preservationPolicy, BODMER_DEPOSIT_FOLDER, DLCMDemoConstants.ZIP_FILES);
      });
    }
  }

  @Test
  void createDlcmDepositsForDemo() {
    final OrganizationalUnit dlcmOrgUnit = this.getOrgUnit(DLCMDemoConstants.DLCM_ORG_UNIT);
    final SubmissionPolicy submissionPolicy = this.submissionPolicyService
            .searchByProperties(Map.of("name", DLCMDemoConstants.DEMO_SUBMISSION_POLICY)).get(0);
    final PreservationPolicy preservationPolicy = this.preservationPolicyService
            .searchByProperties(Map.of("name", DLCMDemoConstants.DEMO_PRESERVATION_POLICY)).get(0);

    assertNotNull(dlcmOrgUnit, "Cannot find orgUnit: " + DLCMDemoConstants.DLCM_ORG_UNIT);
    assertNotNull(submissionPolicy, "Cannot find submission policy: " + DLCMDemoConstants.DEMO_SUBMISSION_POLICY);
    assertNotNull(preservationPolicy, "Cannot find preservation policy: " + DLCMDemoConstants.DEMO_PRESERVATION_POLICY);
    for (final DLCMMetadataVersion version : DLCMMetadataVersion.values()) {
      assertDoesNotThrow(() -> {
        createDeposit(dlcmOrgUnit, version, submissionPolicy, preservationPolicy, DLCM_DEPOSIT_FOLDER,
                DLCMDemoConstants.ZIP_FILES);
      });
    }
  }

  private void createDeposit(
          OrganizationalUnit orgUnit,
          DLCMMetadataVersion version,
          SubmissionPolicy submissionPolicy,
          PreservationPolicy preservationPolicy,
          String depositFolder,
          String fileType) throws IOException {
    log.info("Creating new deposits for {}", version.getVersion());
    for (Resource file : this.listFiles(depositFolder, fileType)) {
      String depositName = this.getFilename(file);
      log.info("Creating '{}' deposit", depositName);
      Deposit deposit = this.createDeposit(orgUnit, version, depositName, depositName, submissionPolicy, preservationPolicy);
      // Add contributors
      this.addContributors(deposit);
      // Add data files
      this.depositService.uploadDepositDataFileArchive(deposit.getResId(), file);
      // Add thumbnail
      if (this.checkIfThumbnailSupported(version)) {
        this.addThumbnail(deposit, this.listFiles(depositFolder, "thumbnail/" + depositName + ".timg")[0]); // TIMG = Thumbnail Image
      }
      this.waitDepositWithDataIsReady(deposit.getResId());
      this.submitAndApproveDeposit(deposit);
      assertTrue(this.waitDepositIsCompleted(deposit.getResId()), "Deposit not completed: " + deposit.getResId());
    }
  }

}
