/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Integration Tests - DepositUserVisitorIT.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.dlcm.test.preingest;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.fail;

import java.util.List;
import java.util.Map;

import org.junit.jupiter.api.Test;
import org.springframework.http.HttpStatus;
import org.springframework.web.client.HttpClientErrorException;

import ch.dlcm.DLCMConstants;
import ch.dlcm.model.preingest.Deposit;
import ch.dlcm.model.security.Role;
import ch.dlcm.model.settings.OrganizationalUnit;
import ch.dlcm.test.DLCMTestConstants;
import ch.dlcm.test.DLCMTestConstants.OrganizationalUnitStatus;

class DepositUserVisitorIT extends AbstractPreingestIT {

  private final String role = Role.VISITOR_ID;

  @Test
  void creationTest() {
    final OrganizationalUnit organizationalUnit = this.getOrCreatePermanentOrganizationalUnit(OrganizationalUnitStatus.OPEN, this.role);
    final String title = DLCMTestConstants.getRandomNameWithTemporaryLabel("deposit created by test");

    // Create a deposit
    final Deposit deposit1 = this.createLocalDeposit(true, title, DEFAULT_DESCRIPTION, this.role);
    try {
      this.depositService.create(deposit1);
      fail("A FORBIDDEN Http response should be received");
    } catch (final HttpClientErrorException e) {
      assertEquals(HttpStatus.FORBIDDEN, e.getStatusCode());
    }

    // Check that the deposit has been created
    final List<Deposit> deposits = this.depositService.searchByProperties(Map.of("title", title,
            DLCMConstants.ORG_UNIT_ID_FIELD, organizationalUnit.getResId()));
    assertEquals(0, deposits.size());
  }

  @Test
  void deleteTest() {
    try {
      this.createRemoteDeposit(true, DEFAULT_TITLE, DEFAULT_DESCRIPTION, this.role);
      fail("An FORBIDDEN Http response should be received");
    } catch (final HttpClientErrorException e) {
      assertEquals(HttpStatus.FORBIDDEN, e.getStatusCode());
    }
  }

  @Test
  void updateTest() {
    try {
      this.depositService.update("test", null);
      fail("An BAD_REQUEST Http response should be received");
    } catch (final HttpClientErrorException e) {
      assertEquals(HttpStatus.BAD_REQUEST, e.getStatusCode());
    }
  }

  @Override
  protected void deleteFixtures() {
  }
}
