/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Integration Tests - PreservationJobFixityIT.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.dlcm.test.preservationplanning;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertTrue;

import java.util.List;

import org.junit.jupiter.api.Test;

import ch.dlcm.model.preservation.AipCopyList;
import ch.dlcm.model.preservation.JobExecution;
import ch.dlcm.model.preservation.JobExecutionReport;
import ch.dlcm.model.preservation.JobRecurrence;
import ch.dlcm.model.preservation.JobType;
import ch.dlcm.model.preservation.PreservationJob;
import ch.dlcm.test.DLCMTestConstants;

class PreservationJobFixityIT extends AbstractPreservationPlanningIT {

  @Override
  protected void setUser() {
    this.restClientTool.sudoAdmin();
  }

  @Test
  void checkAipFixityTest() {

    final String aipFixity = DLCMTestConstants.getRandomNameWithTemporaryLabel("Fixity");

    PreservationJob presJob = new PreservationJob();
    presJob.setName(aipFixity);
    presJob.setJobType(JobType.FIXITY);
    presJob.setJobRecurrence(JobRecurrence.DAILY);

    presJob = this.preservationPlanningService.create(presJob);

    // First execution
    this.preservationPlanningService.start(presJob.getResId());
    assertTrue(this.waitPreservationJobIsCompleted(presJob.getResId()), "Job#1 not completed");

    // Second execution
    this.preservationPlanningService.start(presJob.getResId());
    assertTrue(this.waitPreservationJobIsCompleted(presJob.getResId()), "Job#2 not completed");

    // Check processed items
    final List<JobExecution> jobExeList = this.preservationPlanningService.getJobExecutions(presJob.getResId());
    assertEquals(2, jobExeList.size(), "Wrong number of execution");
    final List<JobExecution> executionList = this.preservationPlanningService.getJobExecutions(presJob.getResId());

    // AIP with status COMPLETED on all storage nodes has to be checked
    final List<AipCopyList> aipCopyLists = this.preservationAIPClientService.findAllCompletedAipCopies();
    int expectedAipCheckedTotal = 0;
    for (AipCopyList aipCopyList : aipCopyLists) {
      expectedAipCheckedTotal += aipCopyList.getCopies().size();
    }

    for (JobExecution jobExe : executionList) {
      final List<JobExecutionReport> reports = this.jobExecutionPlanningService.findReport(presJob.getResId(), jobExe.getResId());
      assertEquals(expectedAipCheckedTotal, reports.get(0).getProcessedItems(),
              "Wrong number of processed items for #" + jobExe.getRunNumber() + " execution");
    }
  }

  @Override
  protected void deleteFixtures() {
    this.clearPreservationJobFixtures();
  }
}
