/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Model - Image.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.dlcm.model;

import javax.persistence.Lob;
import javax.persistence.MappedSuperclass;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;

import com.fasterxml.jackson.annotation.JsonIgnore;

import ch.unige.solidify.rest.ResourceNormalized;

import ch.dlcm.rest.ModuleName;

@MappedSuperclass
public abstract class Image extends ResourceNormalized {

  private static final int DB_FILENAME_LENGTH = 2024;

  @JsonIgnore
  @Lob
  @NotNull
  private byte[] image;

  @Size(max = DB_FILENAME_LENGTH)
  private String fileName;

  private Long fileSize;

  protected Image() {
  }

  public void setImage(byte[] image) {
    this.image = image;
  }

  public void setFileSize(Long fileSize) {
    this.fileSize = fileSize;
  }

  public void setFileName(String fileName) {
    this.fileName = fileName;
  }

  public byte[] getImage() {
    return this.image;
  }

  public Long getFileSize() {
    return this.fileSize;
  }

  public String getFileName() {
    return this.fileName;
  }

  @Override
  public void init() {

  }

  @Override
  public String managedBy() {
    return ModuleName.ADMIN;
  }
}
