/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Model - Package.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.dlcm.model;

import java.util.ArrayList;
import java.util.List;

import ch.unige.solidify.SolidifyConstants;

import ch.dlcm.DLCMConstants;

public enum Package {

  // @formatter:off
  ROOT(""),
  METADATA_FILE(DLCMConstants.METADATA_FILE), 
  METADATA(SolidifyConstants.URL_SEP + DLCMConstants.METADATA_FOLDER),
  RESEARCH(SolidifyConstants.URL_SEP + DLCMConstants.RESEARCH_DATA_FOLDER),
  DOC(SolidifyConstants.URL_SEP + DLCMConstants.DOC_FOLDER), 
  SOFTWARE(SolidifyConstants.URL_SEP + DLCMConstants.SOFTWARE_FOLDER),
  SCHEMAS(SolidifyConstants.URL_SEP + DLCMConstants.SCHEMA_FOLDER),
  INTERNAL(SolidifyConstants.URL_SEP + DLCMConstants.INTERNAL_FOLDER),
  DATASET_THUMBNAIL(DLCMConstants.DATASET_THUMBNAIL),
  UPDATE(SolidifyConstants.URL_SEP + DLCMConstants.UPDATE_FOLDER),
  UPDATE_PACKAGE(DLCMConstants.UPDATE_PACKAGE_FILE);
  // @formatter:on

  public static List<String> getMetadataStructure() {
    final List<String> list = new ArrayList<>();
    list.add(METADATA_FILE.getName());
    list.add(METADATA.getName());
    list.add(SCHEMAS.getName());
    return list;
  }

  private String name;

  private Package(String s) {
    this.name = s;
  }

  public String getName() {
    return this.name;
  }

  public boolean isSubFolder() {
    return this.name.startsWith(SolidifyConstants.URL_SEP);
  }
}
