/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Model - DownloadToken.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.dlcm.model.settings;

import static ch.unige.solidify.SolidifyConstants.DB_ID_LENGTH;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.Transient;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonInclude;

import ch.unige.solidify.rest.ResourceNormalized;

import ch.dlcm.model.security.DownloadTokenType;
import ch.dlcm.rest.ModuleName;

@Entity
public class DownloadToken extends ResourceNormalized {

  @Size(max = DB_ID_LENGTH)
  @Column(length = DB_ID_LENGTH)
  private String userId;

  @Size(max = DB_ID_LENGTH)
  @Column(length = DB_ID_LENGTH)
  private String resourceId;

  @NotNull
  @Enumerated(EnumType.STRING)
  private DownloadTokenType type;

  @JsonIgnore
  private String tokenHash;

  @Transient
  @JsonInclude
  private String token;

  public DownloadToken() {
    // no-op
  }

  public DownloadToken(String userId, DownloadTokenType type, String resourceId, String tokenHash, String token) {
    this.userId = userId;
    this.type = type;
    this.resourceId = resourceId;
    this.tokenHash = tokenHash;
    this.token = token;
  }

  public String getUserId() {
    return this.userId;
  }

  public void setUserId(String userId) {
    this.userId = userId;
  }

  public DownloadTokenType getType() {
    return this.type;
  }

  public void setType(DownloadTokenType type) {
    this.type = type;
  }

  public String getResourceId() {
    return this.resourceId;
  }

  public void setResourceId(String resourceId) {
    this.resourceId = resourceId;
  }

  public String getTokenHash() {
    return this.tokenHash;
  }

  public void setTokenHash(String tokenHash) {
    this.tokenHash = tokenHash;
  }

  public String getToken() {
    return this.token;
  }

  public void setToken(String token) {
    this.token = token;
  }

  @Override
  public void init() {

  }

  @Override
  public String managedBy() {
    return ModuleName.ADMIN;
  }
}
