/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM PreIngest - DepositContributorController.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.dlcm.controller.preingest;

import java.util.List;

import org.springframework.boot.autoconfigure.condition.ConditionalOnBean;
import org.springframework.data.domain.Pageable;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpStatus;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import ch.unige.solidify.SolidifyConstants;
import ch.unige.solidify.controller.AssociationRemoteController;
import ch.unige.solidify.rest.RestCollection;
import ch.unige.solidify.service.CompositeResourceService;

import ch.dlcm.controller.PreIngestController;
import ch.dlcm.model.preingest.Deposit;
import ch.dlcm.model.settings.Person;
import ch.dlcm.rest.ResourceName;
import ch.dlcm.rest.UrlPath;

@PreAuthorize("@depositPermissionService.isAllowed(#parentid, 'GET')")
@RestController
@ConditionalOnBean(PreIngestController.class)
@RequestMapping(UrlPath.PREINGEST_DEPOSIT + SolidifyConstants.URL_PARENT_ID + ResourceName.CONTRIBUTOR)
public class DepositContributorController extends AssociationRemoteController<Deposit, Person> {

  public DepositContributorController(CompositeResourceService<Deposit> itemService) {
    super(itemService, Person.class);
  }

  @Override
  @PreAuthorize("@depositPermissionService.isAllowed(#parentid, 'UPDATE')")
  public HttpEntity<List<Person>> create(@PathVariable String parentid, @RequestBody String[] ids) {
    return super.create(parentid, ids);
  }

  @Override
  public HttpEntity<Person> get(@PathVariable String parentid, @PathVariable String id) {
    return super.get(parentid, id);
  }

  @Override
  public HttpEntity<RestCollection<Person>> list(@PathVariable String parentid, Pageable pageable) {
    return super.list(parentid, pageable);
  }

  @Override
  @PreAuthorize("@depositPermissionService.isAllowed(#parentid, 'UPDATE')")
  public HttpEntity<Person> update(@PathVariable String parentid, @PathVariable String id, @RequestBody Person v2) {
    return super.update(parentid, id, v2);
  }

  @Override
  @PreAuthorize("@depositPermissionService.isAllowed(#parentid, 'UPDATE')")
  public HttpStatus delete(@PathVariable String parentid, @PathVariable String id) {
    return super.delete(parentid, id);
  }

  @Override
  @PreAuthorize("@depositPermissionService.isAllowed(#parentid, 'UPDATE')")
  public HttpStatus deleteList(@PathVariable String parentid, @RequestBody String[] ids) {
    return super.deleteList(parentid, ids);
  }

}
