/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Preservation Planning - ModuleList.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.dlcm.model;

import org.springframework.hateoas.RepresentationModel;

import ch.unige.solidify.IndexConstants;
import ch.unige.solidify.OAIConstants;
import ch.unige.solidify.SolidifyConstants;
import ch.unige.solidify.auth.client.service.AuthorizationClientProperties;

import ch.dlcm.config.DLCMProperties;

public class ModuleList extends RepresentationModel<ModuleList> {
  private String authorization;
  private String admin;
  private String dataManagement;
  private String index;
  private String preingest;
  private String ingest;
  private String access;
  private String oaiInfo;
  private String[] archivalStorage;

  public ModuleList(DLCMProperties dlcmProperties, AuthorizationClientProperties authClientProperties) {
    // Modules with unique instance
    this.authorization = authClientProperties.getPublicAuthorizationServerUrl();
    this.admin = dlcmProperties.getModule().getAdmin().getPublicUrl();
    this.dataManagement = dlcmProperties.getModule().getDataMgmt().getPublicUrl();
    this.index = this.getLinkedModuleUrl(this.dataManagement, IndexConstants.INDEX_MODULE);
    this.preingest = dlcmProperties.getModule().getPreingest().getPublicUrl();
    this.ingest = dlcmProperties.getModule().getIngest().getPublicUrl();
    this.access = dlcmProperties.getModule().getAccess().getPublicUrl();
    this.oaiInfo = this.getLinkedModuleUrl(this.access, OAIConstants.OAI_MODULE);
    // Modules with multiple instances
    this.archivalStorage = dlcmProperties.getArchivalStoragePublicUrls();
  }

  public String getAuthorization() {
    return this.authorization;
  }

  public String getAdmin() {
    return this.admin;
  }

  public String getDataManagement() {
    return this.dataManagement;
  }

  public String getIndex() {
    return this.index;
  }

  public String getPreingest() {
    return this.preingest;
  }

  public String getIngest() {
    return this.ingest;
  }

  public String getAccess() {
    return this.access;
  }

  public String getOaiInfo() {
    return this.oaiInfo;
  }

  public String[] getArchivalStorage() {
    return this.archivalStorage;
  }

  private String getLinkedModuleUrl(String mainModuleUrl, String linkedModuleName) {
    return mainModuleUrl.substring(0, mainModuleUrl.lastIndexOf(SolidifyConstants.URL_SEP) + 1) + linkedModuleName;
  }
}
