/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Resource Server Common - PreservationJobRemoteResourceService.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.dlcm.service.rest.abstractservice;

import ch.unige.solidify.SolidifyConstants;
import ch.unige.solidify.rest.ActionName;
import ch.unige.solidify.service.RemoteResourceService;
import ch.unige.solidify.service.SolidifyRestClientService;

import ch.dlcm.config.DLCMProperties;
import ch.dlcm.model.preservation.PreservationJob;
import ch.dlcm.rest.ResourceName;

public abstract class PreservationJobRemoteResourceService extends RemoteResourceService<PreservationJob> {

  private final String preservationPlanningUrl;

  public PreservationJobRemoteResourceService(DLCMProperties dlcmProperties, SolidifyRestClientService restClientService) {
    super(restClientService);
    this.preservationPlanningUrl = dlcmProperties.getModule().getPreservationPlanning().getUrl();
  }

  public void startPreservationJob(PreservationJob job) {
    String url = this.preservationPlanningUrl
            + SolidifyConstants.URL_SEP + ResourceName.PRES_JOB + SolidifyConstants.URL_SEP
            + job.getResId() + SolidifyConstants.URL_SEP + ActionName.START;
    restClientService.postResourceAction(url);
  }

  @Override
  protected Class<PreservationJob> getResourceClass() {
    return PreservationJob.class;
  }

  @Override
  protected String getResourceUrl() {
    return this.preservationPlanningUrl + SolidifyConstants.URL_SEP + ResourceName.PRES_JOB;
  }

}
