/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Resource Server Common - AbstractPermissionService.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.dlcm.service.security;

import java.util.Arrays;
import java.util.Collection;
import java.util.List;

import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.context.SecurityContextHolder;

import ch.unige.solidify.auth.model.AuthApplicationRole;

import ch.dlcm.DLCMConstants;

public abstract class AbstractPermissionService {

  boolean isRootOrAdminOrTrustedRole() {
    final List<String> authorizedApplicationRoleNames = Arrays.asList(DLCMConstants.getRootAndAdminAndTrustedRoleList());
    return isInApplicationRoleList(authorizedApplicationRoleNames);
  }

  boolean isRootOrTrustedRole() {
    final List<String> authorizedApplicationRoleNames = Arrays.asList(DLCMConstants.getRootAndTrustedRoleList());
    return isInApplicationRoleList(authorizedApplicationRoleNames);
  }

  boolean isAdminRole() {
    final List<String> authorizedApplicationRoleNames = Arrays.asList(AuthApplicationRole.ADMIN_ID);
    return isInApplicationRoleList(authorizedApplicationRoleNames);
  }

  private boolean isInApplicationRoleList(List<String> roleList) {
    final Collection<? extends GrantedAuthority> authorities = SecurityContextHolder.getContext().getAuthentication().getAuthorities();
    for (GrantedAuthority authority : authorities) {
      if (roleList.contains(authority.getAuthority())) {
        return true;
      }
    }
    return false;
  }

}
