/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Resource Server Security - DownloadWithAclPermissionService.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.dlcm.service.security;

import java.security.Principal;
import java.util.List;

import org.springframework.stereotype.Service;

import ch.unige.solidify.service.HttpRequestInfoProvider;

import ch.dlcm.model.settings.ArchiveACL;
import ch.dlcm.service.rest.fallback.FallbackArchiveACLRemoteResourceService;

@Service
public final class DownloadWithAclPermissionService extends AbstractPermissionService {

  private final HttpRequestInfoProvider httpRequestInfoProvider;
  private final FallbackArchiveACLRemoteResourceService archiveACLResourceService;

  public DownloadWithAclPermissionService(HttpRequestInfoProvider httpRequestInfoProvider,
          FallbackArchiveACLRemoteResourceService archiveACLResourceService) {
    this.httpRequestInfoProvider = httpRequestInfoProvider;
    this.archiveACLResourceService = archiveACLResourceService;
  }

  public boolean isAllowed(String aipId) {
    final Principal principal = this.httpRequestInfoProvider.getPrincipal();
    if (principal == null) {
      return false;
    }
    List<ArchiveACL> acls = this.archiveACLResourceService.findMyAcl();
    for (ArchiveACL acl : acls) {
      if (acl.getAipId().equals(aipId) && !acl.isExpired()) {
        return true;
      }
    }
    return false;
  }
}
