/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Solution - AipDoc.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.dlcm.test.resource;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.springframework.restdocs.payload.FieldDescriptor;

import ch.unige.solidify.rest.ActionName;
import ch.unige.solidify.test.helper.AbstractResourceDoc;
import ch.unige.solidify.test.helper.FieldDescriptorBuilder;
import ch.unige.solidify.test.model.TypeField;

import ch.dlcm.model.oais.ArchiveContainer;
import ch.dlcm.rest.DLCMActionName;
import ch.dlcm.rest.ResourceName;
import ch.dlcm.rest.UrlPath;

public class AipDoc extends AbstractResourceDoc {

  @Override
  public List<String> getListLinks() {
    return Arrays.asList(ResourceName.SCHEMA, ResourceName.PROFILE, ActionName.VALUES, DLCMActionName.ARCHIVAL_COLLECTIONS,
            DLCMActionName.ARCHIVAL_UNITS);
  }

  @Override
  public String getName() {
    return UrlPath.ARCHIVAL_STORAGE_AIP;
  }

  @Override
  public List<String> getResourceLinks() {
    return Arrays.asList(ResourceName.DATAFILE, ActionName.UPLOAD, ActionName.RESUME, ActionName.HISTORY);
  }

  @Override
  protected List<FieldDescriptor> getDocumentationFields() {

    // @formatter:off
    final List<FieldDescriptor> documentationFields = new ArrayList<>(Arrays.asList(
      new FieldDescriptorBuilder(this)
        .name("archiveSize")
        .description("The size in bytes of the AIP")
        .type(TypeField.INT32)
        .build(),
      new FieldDescriptorBuilder(this)
        .name("archiveFileNumber")
        .description("The total number of file in the AIP")
        .type(TypeField.INT32)
        .build(),
      new FieldDescriptorBuilder(this)
        .name("smartSize")
        .description("The size in human-readable format of the AIP")
        .type(TypeField.STRING)
        .build(),
      new FieldDescriptorBuilder(this)
        .name("tombstoneSize")
        .description("The size in bytes of the tombstone AIP")
        .type(TypeField.INT32)
        .build(),
      new FieldDescriptorBuilder(this)
        .name("smartTombstoneSize")
        .description("The size in human-readable format of the tombstone AIP")
        .type(TypeField.STRING)
        .build(),
      new FieldDescriptorBuilder(this)
        .name("checksums")
        .description("The checksum list of the AIP")
        .type(TypeField.ARRAY)
        .build(),
      new FieldDescriptorBuilder(this)
        .name("archiveContainer")
        .description("The container type of the AIP")
        .mandatory(true)
        .enumValues(this.enumToList(ArchiveContainer.values()))
        .testValue(ArchiveContainer.BAG_IT.toString())
        .build(),
      new FieldDescriptorBuilder(this)
        .name("archivalUnit")
        .description("if the AIP is an unit (AIU) or a collection (AIC)")
        .type(TypeField.BOOLEAN)
        .build(),
      new FieldDescriptorBuilder(this)
        .name("sipIds")
        .description("The source SIP IDs of the AIP")
        .type(TypeField.ARRAY)
        .build(),
      new FieldDescriptorBuilder(this)
        .name("archiveId")
        .description("The storage URL of the AIP")
        .type(TypeField.STRING)
        .build(),
      new FieldDescriptorBuilder(this)
        .name("retention")
        .description("The retention duration in days of the AIP")
        .mandatory(true)
        .type(TypeField.INT32)
        .customConstraints(CONSTRAINT_DATE_INTEGER)
        .build(),
      new FieldDescriptorBuilder(this)
        .name("smartRetention")
        .description("The retention duration in human-readable format of the AIP")
        .build(),
      new FieldDescriptorBuilder(this)
        .name("retentionEnd")
        .description("The end of the AIP retention")
        .type(TypeField.DATE_TIME)
        .build(),
      new FieldDescriptorBuilder(this)
        .name("dispositionApproval")
        .description("Mandatory step to approve the AIP disposition")
        .type(TypeField.BOOLEAN)
        .build(),
      new FieldDescriptorBuilder(this)
        .name("ready")
        .description("If AIP package is ready")
        .type(TypeField.BOOLEAN)
        .build(),
      new FieldDescriptorBuilder(this)
        .name("dataFileNumber")
        .description("Number of AIP data files")
        .type(TypeField.INT32)
        .build(),
      new FieldDescriptorBuilder(this)
        .name("collectionSize")
        .description("Number of AIPs in collection")
        .type(TypeField.INT32)
        .build()));
    // @formatter:on
    documentationFields.addAll(new InformationPackageDoc().getDocumentationFields());
    documentationFields.addAll(new FileFormatDoc().getDocumentationFields());
    documentationFields.addAll(new VirusCheckDoc().getDocumentationFields());
    documentationFields.addAll(new ChecksumCheckDoc().getDocumentationFields());
    return documentationFields;
  }
}
