/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Solution - ArchiveACLDoc.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.dlcm.test.resource;

import static org.springframework.restdocs.payload.PayloadDocumentation.subsectionWithPath;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import org.springframework.restdocs.payload.FieldDescriptor;
import org.springframework.restdocs.snippet.Attributes;

import ch.unige.solidify.test.helper.AbstractResourceDoc;
import ch.unige.solidify.test.helper.FieldDescriptorBuilder;
import ch.unige.solidify.test.model.TypeField;

import ch.dlcm.DLCMConstants;
import ch.dlcm.rest.UrlPath;

public class ArchiveACLDoc extends AbstractResourceDoc {

  @Override
  public List<String> getListLinks() {
    return Collections.emptyList();
  }

  @Override
  public String getName() {
    return UrlPath.ADMIN_ARCHIVE_ACL;
  }

  @Override
  public List<String> getResourceLinks() {
    return Collections.emptyList();
  }

  @Override
  protected List<FieldDescriptor> getDocumentationFields() {

    // @formatter:off
    return new ArrayList<>(Arrays.asList(
      new FieldDescriptorBuilder(this)
        .name("aipId")
        .description("The ID of the archive ACL")
        .mandatory(true)
        .testValue(DLCMConstants.TEST_RES_ID)
        .build(),
      new FieldDescriptorBuilder(this)
        .name("user.resId")
        .description("The user ID of the archive ACL")
        .mandatory(true)
        .type(TypeField.STRING)
        .testValue(DLCMConstants.TEST_RES_ID)
        .build(),
      subsectionWithPath("user")
        .description("The user of the archive ACL")
        .attributes(new Attributes.Attribute(READ_ONLY_VALUE, String.valueOf(true))),
      new FieldDescriptorBuilder(this)
        .name("organizationalUnit.resId")
        .description("The organizational unit ID of the archive ACL")
        .mandatory(true)
        .type(TypeField.STRING)
        .testValue(DLCMConstants.TEST_RES_ID)
        .build(),
      subsectionWithPath("organizationalUnit")
        .description("The organizational unit of the archive ACL")
        .attributes(new Attributes.Attribute(READ_ONLY_VALUE, String.valueOf(true))),
      new FieldDescriptorBuilder(this)
        .name("expiration")
        .description("The expiration date of the archive ACL")
        .type(TypeField.DATE_TIME)
        .build(),
      new FieldDescriptorBuilder(this)
        .name("expired")
        .description("if the the archive ACL is expired")
        .type(TypeField.BOOLEAN)
        .build()));
    // @formatter:on
  }
}
