/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Solution - ArchiveTypeDoc.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.dlcm.test.resource;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import org.springframework.restdocs.payload.FieldDescriptor;
import org.springframework.restdocs.payload.JsonFieldType;

import ch.unige.solidify.rest.ActionName;
import ch.unige.solidify.test.helper.AbstractResourceDoc;
import ch.unige.solidify.test.helper.FieldDescriptorBuilder;
import ch.unige.solidify.test.model.TypeField;

import ch.dlcm.rest.DLCMActionName;
import ch.dlcm.rest.UrlPath;

public class ArchiveTypeDoc extends AbstractResourceDoc {

  @Override
  public List<String> getListLinks() {
    return Arrays.asList(ActionName.VALUES, DLCMActionName.LIST_ARCHIVE_MASTER_TYPES, DLCMActionName.LIST_ARCHIVE_TYPES_OF_MASTER_TYPE);
  }

  @Override
  public String getName() {
    return UrlPath.ADMIN_ARCHIVE_TYPE;
  }

  @Override
  public List<String> getResourceLinks() {
    return Collections.emptyList();
  }

  @Override
  protected List<FieldDescriptor> getDocumentationFields() {
    // @formatter:off
    return Arrays.asList(
      new FieldDescriptorBuilder(this)
        .name("masterType")
        .description("The master type of the archive type")
        .type(TypeField.UNDEFINED)
        .build(),
        new FieldDescriptorBuilder(this)
        .name("typeName")
        .description("The name of the archive type")
        .mandatory(true)
        .type(TypeField.STRING)
        .testValue("Other")
        .build().type(JsonFieldType.STRING),
        new FieldDescriptorBuilder(this)
        .name("masterArchiveType")
        .description("If the archive type is master type")
        .type(TypeField.BOOLEAN)
        .build());
    // @formatter:on
  }
}
