/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Solution - ContributorDoc.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.dlcm.test.resource;

import java.util.Arrays;
import java.util.List;

import org.springframework.restdocs.payload.FieldDescriptor;

import ch.unige.solidify.test.helper.AbstractResourceDoc;
import ch.unige.solidify.test.helper.FieldDescriptorBuilder;
import ch.unige.solidify.test.model.TypeField;

import ch.dlcm.rest.DLCMActionName;
import ch.dlcm.rest.ResourceName;
import ch.dlcm.rest.UrlPath;

public class ContributorDoc extends AbstractResourceDoc {

  @Override
  public List<String> getListLinks() {
    return Arrays.asList(DLCMActionName.CONTRIBUTOR_WITH_DEPOSITS);
  }

  @Override
  public String getName() {
    return UrlPath.PREINGEST_CONTRIBUTOR;
  }

  @Override
  public List<String> getResourceLinks() {
    return Arrays.asList(ResourceName.DEPOSIT);
  }

  @Override
  protected List<FieldDescriptor> getDocumentationFields() {
    // @formatter:off
    return Arrays.asList(
      new FieldDescriptorBuilder(this)
        .name("firstName")
        .description("The first name of the contributor")
        .mandatory(true)
        .type(TypeField.STRING)
        .build(),
      new FieldDescriptorBuilder(this)
        .name("lastName")
        .description("The last name of the contributor")
        .mandatory(true)
        .type(TypeField.STRING)
        .build(),
      new FieldDescriptorBuilder(this)
        .name("fullName")
        .description("The full name of the contributor")
        .type(TypeField.STRING)
        .build(),
      new FieldDescriptorBuilder(this)
        .name("orcid")
        .description("The ORCID of the contributor")
        .type(TypeField.STRING)
        .customConstraints(this.displayFormat("xxxx-xxxx-xxxx-xxxx"))
        .testValue(TEST_ORCID)
        .build(),
      new FieldDescriptorBuilder(this)
        .name("verifiedOrcid")
        .description("if the contributor ORCID is verified")
        .type(TypeField.BOOLEAN)
        .build(),
      new FieldDescriptorBuilder(this)
        .name("depositNumber")
        .description("The deposit number of the contributor")
        .type(TypeField.INT32)
        .build(),
      new FieldDescriptorBuilder(this)
        .name("avatar")
        .description("Avatar of the person")
        .type(TypeField.UNDEFINED)
        .build());
    // @formatter:on
  }

}
