/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Solution - DipDoc.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.dlcm.test.resource;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.springframework.restdocs.payload.FieldDescriptor;

import ch.unige.solidify.rest.ActionName;
import ch.unige.solidify.test.helper.AbstractResourceDoc;
import ch.unige.solidify.test.helper.FieldDescriptorBuilder;
import ch.unige.solidify.test.model.TypeField;

import ch.dlcm.rest.ResourceName;
import ch.dlcm.rest.UrlPath;

public class DipDoc extends AbstractResourceDoc {

  @Override
  public List<String> getListLinks() {
    return Arrays.asList(ResourceName.SCHEMA, ResourceName.PROFILE, ActionName.VALUES);
  }

  @Override
  public String getName() {
    return UrlPath.ACCESS_DIP;
  }

  @Override
  public List<String> getResourceLinks() {
    return Arrays.asList(ResourceName.DATAFILE, ActionName.HISTORY, ResourceName.AIP);
  }

  @Override
  protected List<FieldDescriptor> getDocumentationFields() {
    // @formatter:off
    final List<FieldDescriptor> documentationFields = new ArrayList<>(Arrays.asList(
      new FieldDescriptorBuilder(this)
        .name("aipIds")
        .description("The source AIP IDs of the DIP")
        .type(TypeField.ARRAY)
        .build(),
      new FieldDescriptorBuilder(this)
        .name("ready")
        .description("If DIP package is ready")
        .type(TypeField.BOOLEAN)
        .build()));
    // @formatter:on
    documentationFields.addAll(new InformationPackageDoc().getDocumentationFields());
    return documentationFields;
  }

}
