/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Solution - EmbargoInfoDoc.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.dlcm.test.resource;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import org.springframework.restdocs.payload.FieldDescriptor;
import org.springframework.restdocs.payload.JsonFieldType;

import ch.unige.solidify.test.helper.AbstractResourceDoc;
import ch.unige.solidify.test.helper.FieldDescriptorBuilder;
import ch.unige.solidify.test.model.TypeField;

import ch.dlcm.model.Access;

public class EmbargoInfoDoc extends AbstractResourceDoc {

  @Override
  public List<String> getListLinks() {
    return Collections.emptyList();
  }

  @Override
  public String getName() {
    return "embargo-info";
  }

  @Override
  public List<String> getResourceLinks() {
    return Collections.emptyList();
  }

  @Override
  protected List<FieldDescriptor> getDocumentationFields() {
    return this.getDocumentationFields(null);
  }

  protected List<FieldDescriptor> getDocumentationFields(String prefix) {
    if (prefix == null) {
      prefix = "";
    }
    // @formatter:off
    return Arrays.asList(
      new FieldDescriptorBuilder(this)
        .name(prefix + "embargo.startDate")
        .description("Starting date of embargo period")
        .type(TypeField.DATE_TIME)
        .build(),
      new FieldDescriptorBuilder(this)
        .name(prefix + "embargo.months")
        .description("Month number of embargo duration")
        .type(TypeField.INT32)
        .build().type(JsonFieldType.NUMBER),
      new FieldDescriptorBuilder(this)
        .name(prefix + "embargo.access")
        .description("Starting date of embargo period")
        .enumValues(this.enumToList(Access.values()))
        .testValue(Access.RESTRICTED.toString())
        .build());
    // @formatter:off
  }
}
