/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Solution - InformationPackageDoc.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.dlcm.test.resource;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import org.springframework.restdocs.payload.FieldDescriptor;

import ch.unige.solidify.test.helper.AbstractResourceDoc;
import ch.unige.solidify.test.helper.FieldDescriptorBuilder;
import ch.unige.solidify.test.model.TypeField;

import ch.dlcm.DLCMMetadataVersion;
import ch.dlcm.model.Access;
import ch.dlcm.model.ComplianceLevel;
import ch.dlcm.model.DataTag;
import ch.dlcm.model.PackageStatus;

public class InformationPackageDoc extends AbstractResourceDoc {

  @Override
  public List<String> getListLinks() {
    return Collections.emptyList();
  }

  @Override
  public String getName() {
    return "information-package";
  }

  @Override
  public List<String> getResourceLinks() {
    return Collections.emptyList();
  }

  @Override
  protected List<FieldDescriptor> getDocumentationFields() {
    // @formatter:off
    final List<FieldDescriptor> documentationFields =  new ArrayList<>(Arrays.asList(
      new FieldDescriptorBuilder(this)
        .name("info.name")
        .description("The name of the package")
        .mandatory(true)
        .type(TypeField.STRING)
        .build(),
      new FieldDescriptorBuilder(this)
        .name("info.organizationalUnitId")
        .description("The organizational Unit ID of the information package")
        .mandatory(true)
        .type(TypeField.STRING)
        .build(),
      new FieldDescriptorBuilder(this)
        .name("info.description")
        .description("The description of the package")
        .type(TypeField.STRING)
        .build(),
      new FieldDescriptorBuilder(this)
        .name("info.access")
        .description("The final access level of the package")
        .enumValues(this.enumToList(Access.values()))
        .testValue(Access.RESTRICTED.toString())
        .build(),
      new FieldDescriptorBuilder(this)
        .name("info.currentAccess")
        .description("The current access level, deducted from the final access level and an eventual embargo")
        .build(),
      new FieldDescriptorBuilder(this)
        .name("info.accessCurrentlyPublic")
        .description("A boolean indicating wether the current access level is PUBLIC")
        .type(TypeField.BOOLEAN)
        .build(),
      new FieldDescriptorBuilder(this)
        .name("info.containsUpdatedMetadata")
        .description("If the deposit contains updated metadata")
        .type(TypeField.BOOLEAN)
        .build(),
      new FieldDescriptorBuilder(this)
        .name("info.dataSensitivity")
        .description("The data sensitivity of the deposit")
        .enumValues(this.enumToList(DataTag.values()))
        .testValue(DataTag.GREEN.toString())
        .build(),  
      new FieldDescriptorBuilder(this)
        .name("info.licenseId")
        .description("The license ID of the package")
        .type(TypeField.STRING)
        .customConstraints(CONSTRAINT_VALID_ID)
        .build(),
      new FieldDescriptorBuilder(this)
        .name("info.complianceLevel")
        .description("The compliance level of the package")
        .enumValues(this.enumToList(ComplianceLevel.values()))
        .testValue(ComplianceLevel.NOT_ASSESSED.toString())
        .build(),
      new FieldDescriptorBuilder(this)
        .name("info.status")
        .description("The status of the package")
        .enumValues(this.enumToList(PackageStatus.values()))
        .testValue(NO_TEST_VALUE)
        .build(),
      new FieldDescriptorBuilder(this)
        .name("info.statusMessage")
        .description("The message related to the status")
        .type(TypeField.STRING)
        .build(),
      new FieldDescriptorBuilder(this)
        .name("info.metadataVersion")
        .description("The metadata version of the deposit")
        .enumValues( this.enumToList(DLCMMetadataVersion.values()))
        .testValue(DLCMMetadataVersion.getDefaultVersion().getVersion())
        .build()));
    // @formatter:off
    documentationFields.addAll(new EmbargoInfoDoc().getDocumentationFields("info."));
    return documentationFields;
  }

}
