/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Solution - JobSchedulingDoc.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.dlcm.test.resource;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import org.springframework.restdocs.payload.FieldDescriptor;

import ch.unige.solidify.test.helper.AbstractResourceDoc;
import ch.unige.solidify.test.helper.FieldDescriptorBuilder;
import ch.unige.solidify.test.model.TypeField;

public class JobSchedulingDoc extends AbstractResourceDoc {

  @Override
  public List<String> getListLinks() {
    return Collections.emptyList();
  }

  @Override
  public String getName() {
    return "scheduling";
  }

  @Override
  public List<String> getResourceLinks() {
    return Collections.emptyList();
  }

  @Override
  protected List<FieldDescriptor> getDocumentationFields() {
    // @formatter:off
    return new ArrayList<>(Arrays.asList(
      new FieldDescriptorBuilder(this)
        .name("scheduling.hour")
        .description("The hour to schedule a daily/weekly/monthly/yearly job")
        .mandatory(false)
        .type(TypeField.INT32)
        .customConstraints("Between 0 and 23")
        .build(),
      new FieldDescriptorBuilder(this)
        .name("scheduling.weekDay")
        .description("The day of the week to schedule a weekly/monthly/yearly job")
        .mandatory(false)
        .type(TypeField.INT32)
        .customConstraints("1 for Monday, 2 for Tuesday, 3 for Wednesday, 4 for Thursday, 5 for Friday, 6 for Saturday, 7 for Sunday")
        .build(),
      new FieldDescriptorBuilder(this)
        .name("scheduling.monthDay")
        .description("The day of the month to schedule a monthly/yearly job")
        .mandatory(false)
        .type(TypeField.INT32)
        .customConstraints("Between 1 and 31")
        .build(),
      new FieldDescriptorBuilder(this)
        .name("scheduling.month")
        .description("The month to schedule a yearly job")
        .mandatory(false)
        .type(TypeField.INT32)
        .customConstraints("1 for January, 2 for February, 3 for March, 4 for April, 5 for May, 6 for June, 7 for July, 8 for August, 9 for September, 10 for October, 11 for November, 12 for december")
        .build()));
    // @formatter:off
  }

}
