/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Solution - OrganizationalUnitDoc.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.dlcm.test.resource;

import static org.springframework.restdocs.payload.PayloadDocumentation.subsectionWithPath;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import org.springframework.restdocs.payload.FieldDescriptor;
import org.springframework.restdocs.payload.JsonFieldType;
import org.springframework.restdocs.snippet.Attributes;

import ch.unige.solidify.rest.ActionName;
import ch.unige.solidify.test.helper.AbstractResourceDoc;
import ch.unige.solidify.test.helper.FieldDescriptorBuilder;
import ch.unige.solidify.test.model.TypeField;

import ch.dlcm.rest.DLCMActionName;
import ch.dlcm.rest.ResourceName;
import ch.dlcm.rest.UrlPath;

public class OrganizationalUnitDoc extends AbstractResourceDoc {

  @Override
  public List<String> getListLinks() {
    return Collections.emptyList();
  }

  @Override
  public String getName() {
    return UrlPath.ADMIN_UNIT;
  }

  @Override
  public List<String> getResourceLinks() {
    return Arrays.asList(ResourceName.PERSON, ResourceName.INSTITUTION, ResourceName.FUNDING_AGENCY, ResourceName.PRES_POLICY,
            ResourceName.SUB_POLICY, ResourceName.DIS_POLICY, ActionName.CLOSE, ResourceName.ADDITIONAL_FIELDS_FORMS, ResourceName.ARCHIVE_ACL,
            DLCMActionName.UPLOAD_LOGO, DLCMActionName.DOWNLOAD_LOGO);
  }

  @Override
  protected List<FieldDescriptor> getDocumentationFields() {
    return new ArrayList<>(Arrays.asList(
    // @formatter:off
      new FieldDescriptorBuilder(this)
        .name("name")
        .description("The name of the organizational unit")
        .mandatory(true)
        .type(TypeField.STRING)
        .testValue("The name of the organizational unit")
        .build(),
      new FieldDescriptorBuilder(this)
        .name("description")
        .description("The description of the institution")
        .type(TypeField.STRING)
        .build().type(JsonFieldType.STRING),
      new FieldDescriptorBuilder(this)
        .name("url")
        .description("The URL of the organizational unit")
        .type(TypeField.URL)
        .customConstraints(CONSTRAINT_VALID_URL)
        .testValue(TEST_URL_VALUE)
        .build().type(JsonFieldType.STRING),
      new FieldDescriptorBuilder(this)
        .name("defaultSubmissionPolicy")
        .description("The default submission policy of the organizational unit")
        .type(TypeField.UNDEFINED)
        .build(),
      new FieldDescriptorBuilder(this)
        .name("defaultPreservationPolicy")
        .description("The default preservation policy of the organizational unit")
        .type(TypeField.UNDEFINED)
        .build(),
      new FieldDescriptorBuilder(this)
        .name("defaultDisseminationPolicy")
        .description("The default dissemination policy of the organizational unit")
        .type(TypeField.UNDEFINED)
        .build(),
      new FieldDescriptorBuilder(this)
        .name("openingDate")
        .description("The opening date of the organizational unit")
        .type(TypeField.DATE)
        .build(),
      new FieldDescriptorBuilder(this)
        .name("closingDate")
        .description("The closing date of the organizational unit")
        .type(TypeField.DATE)
        .build(),
      new FieldDescriptorBuilder(this)
        .name("open")
        .description("Indicates whether the organizational unit is currently open")
        .type(TypeField.BOOLEAN)
        .build(),
      new FieldDescriptorBuilder(this)
        .name("keywords")
        .description("The keywords associated with this organizational unit, they are used as default for deposits created within this organizational unit")
        .type(TypeField.ARRAY)
        .build(),
      new FieldDescriptorBuilder(this)
        .name("logo")
        .description("The logo of the organizational unit")
        .type(TypeField.UNDEFINED)
        .build(),
      subsectionWithPath("defaultLicense")
        .description("Default license for this organizational unit")
        .attributes(new Attributes.Attribute(READ_ONLY_VALUE, String.valueOf(true))),
      new FieldDescriptorBuilder(this)
        .name("isEmpty")
        .description("Indicates whether the organizational unit contains any data (depo, sip, aip)")
        .type(TypeField.BOOLEAN)
        .build()));
    // @formatter:on
  }
}
