/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Solution - PersonDoc.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.dlcm.test.resource;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import org.springframework.restdocs.payload.FieldDescriptor;
import org.springframework.restdocs.payload.JsonFieldType;

import ch.unige.solidify.test.helper.AbstractResourceDoc;
import ch.unige.solidify.test.helper.FieldDescriptorBuilder;
import ch.unige.solidify.test.model.TypeField;

import ch.dlcm.model.IdentifierType;
import ch.dlcm.model.settings.Person;
import ch.dlcm.rest.DLCMActionName;
import ch.dlcm.rest.ResourceName;
import ch.dlcm.rest.UrlPath;

public class PersonDoc extends AbstractResourceDoc {

  @Override
  public List<String> getListLinks() {
    return Collections.emptyList();
  }

  @Override
  public String getModelName() {
    return getCamelCase(Person.class.getSimpleName());
  }

  @Override
  public String getName() {
    return UrlPath.ADMIN_PERSON;
  }

  @Override
  public List<String> getResourceLinks() {
    return Arrays.asList(DLCMActionName.UPLOAD_AVATAR, DLCMActionName.DOWNLOAD_AVATAR, ResourceName.INSTITUTION, ResourceName.ORG_UNIT,
            IdentifierType.ORCID.toString().toLowerCase());
  }

  @Override
  protected List<FieldDescriptor> getDocumentationFields() {
    // @formatter:off
    return Arrays.asList(
      new FieldDescriptorBuilder(this)
        .name("firstName")
        .description("The first name of the person")
        .mandatory(true)
        .type(TypeField.STRING)
        .build(),
      new FieldDescriptorBuilder(this)
        .name("lastName")
        .description("The last name of the person")
        .mandatory(true)
        .type(TypeField.STRING)
        .build(),
      new FieldDescriptorBuilder(this)
        .name("fullName")
        .description("The full name of the person")
        .type(TypeField.STRING)
        .build(),
      new FieldDescriptorBuilder(this)
        .name("avatar")
        .description("Avatar of the person")
        .type(TypeField.UNDEFINED)
        .build(),
      new FieldDescriptorBuilder(this)
        .name("orcid")
        .description("The ORCID of the person")
        .type(TypeField.STRING)
        .customConstraints(this.displayFormat("xxxx-xxxx-xxxx-xxxx"))
        .testValue(TEST_ORCID)
        .build().type(JsonFieldType.STRING),
      new FieldDescriptorBuilder(this)
        .name("verifiedOrcid")
        .description("if the person ORCID is verified")
        .type(TypeField.BOOLEAN)
        .build());
    // @formatter:on
  }

}
