/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Solution - PreservationPolicyDoc.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.dlcm.test.resource;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import org.springframework.restdocs.payload.FieldDescriptor;

import ch.unige.solidify.test.helper.AbstractResourceDoc;
import ch.unige.solidify.test.helper.FieldDescriptorBuilder;
import ch.unige.solidify.test.model.TypeField;

import ch.dlcm.rest.UrlPath;

public class PreservationPolicyDoc extends AbstractResourceDoc {

  @Override
  public List<String> getListLinks() {
    return Collections.emptyList();
  }

  @Override
  public String getName() {
    return UrlPath.ADMIN_PRESERVATION_POLICY;
  }

  @Override
  public List<String> getResourceLinks() {
    return Collections.emptyList();
  }

  @Override
  protected List<FieldDescriptor> getDocumentationFields() {
    return Arrays.asList(
    // @formatter:
      new FieldDescriptorBuilder(this)
        .name("name")
        .description("The name of the preservation policy")
        .mandatory(true)
        .type(TypeField.STRING)
        .build(),
      new FieldDescriptorBuilder(this)
        .name("retention")
        .description("The retention duration in days of the preservation policy")
        .mandatory(true)
        .type(TypeField.INT32)
        .customConstraints(CONSTRAINT_DATE_INTEGER)
        .build(),
      new FieldDescriptorBuilder(this)
        .name("smartRetention")
        .description("The retention duration in human-readable format of the preservation policy")
        .type(TypeField.STRING)
        .build(),
      new FieldDescriptorBuilder(this)
        .name("dispositionApproval")
        .description("Mandatory step to approve the AIP disposition")
        .type(TypeField.BOOLEAN)
        .build(),
      new FieldDescriptorBuilder(this)
        .name("useNumber")
        .description("Use number in organizational units of the preservation policy")
        .type(TypeField.INT32)
        .build());
    // @formatter:on
  }
}
